#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.links.zmi module

"""

from pyramid.view import view_config
from zope.interface import implementer

from pyams_content.component.association.interfaces import IAssociationContainer, IAssociationContainerTarget, \
    IAssociationInfo
from pyams_content.component.association.zmi import AssociationItemAddFormMixin, \
    AssociationItemAddMenuMixin
from pyams_content.component.association.zmi.interfaces import IAssociationsTable
from pyams_content.component.links import ExternalLink, InternalLink, MailtoLink
from pyams_content.component.links.interfaces import IBaseLink, IExternalLink, IInternalLink, \
    ILinkContainerTarget, IMailtoLink
from pyams_content.component.links.zmi.interfaces import ILinkAddForm, ILinkEditForm
from pyams_content.reference.pictogram.zmi.widget import PictogramSelectFieldWidget
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_security.security import ProtectedViewObjectMixin
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager
from pyams_zmi.utils import get_object_hint, get_object_label

__docformat__ = 'restructuredtext'

from pyams_content import _


@implementer(ILinkAddForm)
class LinkAddFormMixin(AssociationItemAddFormMixin):
    """Link add form mixin class"""

    legend = _("New link properties")


@adapter_config(required=(IAssociationContainer, IAdminLayer, ILinkAddForm),
                provides=IFormTitle)
def link_add_form_title(context, request, form):
    """Link add form title getter"""
    parent = get_parent(context, IAssociationContainerTarget)
    hint = get_object_hint(parent, request, form)
    label = get_object_label(parent, request, form)
    return TITLE_SPAN_BREAK.format(hint, label)


@implementer(ILinkEditForm)
class LinkEditFormMixin:
    """Link edit form mixin class"""

    @property
    def subtitle(self):
        translate = self.request.localizer.translate
        return translate(_("{}: {}")).format(
            translate(self.context.icon_hint),
            get_object_label(self.context, self.request, self))

    legend = _("Link properties")


@adapter_config(required=(IBaseLink, IAdminLayer, ILinkEditForm),
                provides=IFormTitle)
def link_edit_form_title(context, request, form):
    """Link edit form title"""
    parent = get_parent(context, IAssociationContainerTarget)
    hint = get_object_hint(parent, request, form)
    label = get_object_label(parent, request, form)
    return TITLE_SPAN_BREAK.format(
        hint, label)


#
# Internal links management
#

@viewlet_config(name='add-internal-link.menu',
                context=ILinkContainerTarget, layer=IAdminLayer, view=IAssociationsTable,
                manager=IContextAddingsViewletManager, weight=50)
class InternalLinkAddMenu(ProtectedViewObjectMixin, AssociationItemAddMenuMixin, MenuItem):
    """Internal link add menu"""

    label = InternalLink.icon_hint
    icon_class = InternalLink.icon_class

    href = 'add-internal-link.html'


@ajax_form_config(name='add-internal-link.html',
                  context=IAssociationContainer, layer=IPyAMSLayer)
class InternalLinkAddForm(LinkAddFormMixin, AdminModalAddForm):
    """Internal link add form"""

    subtitle = _("New internal link")
    legend = _("New internal link properties")

    fields = Fields(IInternalLink).select('reference', 'force_canonical_url', 'title',
                                          'description', 'pictogram_name')
    fields['pictogram_name'].widget_factory = PictogramSelectFieldWidget
    content_factory = IInternalLink


@ajax_form_config(name='properties.html',
                  context=IInternalLink, layer=IPyAMSLayer)
class InternalLinkPropertiesEditForm(LinkEditFormMixin, AdminModalEditForm):
    """internal link properties edit form"""

    fields = Fields(IInternalLink).select('reference', 'force_canonical_url', 'title',
                                          'description', 'pictogram_name')
    fields['pictogram_name'].widget_factory = PictogramSelectFieldWidget


#
# External links management
#

@viewlet_config(name='add-external-link.menu',
                context=ILinkContainerTarget, layer=IAdminLayer, view=IAssociationsTable,
                manager=IContextAddingsViewletManager, weight=55)
class ExternalLinkAddMenu(ProtectedViewObjectMixin, AssociationItemAddMenuMixin, MenuItem):
    """External link add menu"""

    label = ExternalLink.icon_hint
    icon_class = ExternalLink.icon_class

    href = 'add-external-link.html'


@ajax_form_config(name='add-external-link.html',
                  context=IAssociationContainer, layer=IPyAMSLayer)
class ExternalLinkAddForm(LinkAddFormMixin, AdminModalAddForm):
    """External link add form"""

    subtitle = _("New external link")
    legend = _("New external link properties")

    fields = Fields(IExternalLink).select('url', 'title', 'description',
                                          'language', 'pictogram_name')
    fields['pictogram_name'].widget_factory = PictogramSelectFieldWidget
    content_factory = IExternalLink


@ajax_form_config(name='properties.html',
                  context=IExternalLink, layer=IPyAMSLayer)
class ExternalLinkPropertiesEditForm(LinkEditFormMixin, AdminModalEditForm):
    """external link properties edit form"""

    fields = Fields(IExternalLink).select('url', 'title', 'description',
                                          'language', 'pictogram_name')
    fields['pictogram_name'].widget_factory = PictogramSelectFieldWidget


#
# Mailto links management
#

@viewlet_config(name='add-mailto-link.menu',
                context=ILinkContainerTarget, layer=IAdminLayer, view=IAssociationsTable,
                manager=IContextAddingsViewletManager, weight=60)
class MailtoLinkAddMenu(ProtectedViewObjectMixin, AssociationItemAddMenuMixin, MenuItem):
    """Mailto link add menu"""

    label = MailtoLink.icon_hint
    icon_class = MailtoLink.icon_class

    href = 'add-mailto-link.html'


@ajax_form_config(name='add-mailto-link.html',
                  context=IAssociationContainer, layer=IPyAMSLayer)
class MailtoLinkAddForm(LinkAddFormMixin, AdminModalAddForm):
    """Mailto link add form"""

    subtitle = _("New mailto link")
    legend = _("New mailto link properties")

    fields = Fields(IMailtoLink).select('address', 'address_name', 'title',
                                        'description', 'pictogram_name')
    fields['pictogram_name'].widget_factory = PictogramSelectFieldWidget
    content_factory = IMailtoLink


@ajax_form_config(name='properties.html',
                  context=IMailtoLink, layer=IPyAMSLayer)
class MailtoLinkPropertiesEditForm(LinkEditFormMixin, AdminModalEditForm):
    """mailto link properties edit form"""

    fields = Fields(IMailtoLink).select('address', 'address_name', 'title',
                                        'description', 'pictogram_name')
    fields['pictogram_name'].widget_factory = PictogramSelectFieldWidget


#
# Links getter API
#

@view_config(name='get-links-list.json',
             request_type=IPyAMSLayer,
             permission=VIEW_SYSTEM_PERMISSION,
             renderer='json', xhr=True)
def get_links_list(request):
    """Links list getter"""
    result = []
    context = request.context
    links = IAssociationContainer(context, None)
    if not links:
        return result
    for link in links.values():
        if not (IInternalLink.providedBy(link) or IExternalLink.providedBy(link)):
            continue
        link_info = IAssociationInfo(link)
        result.append({
            'title': link_info.user_title,
            'value': link.get_editor_url()
        })
    return result



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.navigation.zmi module

"""

from zope.interface import Interface, implementer

from pyams_content.component.association.zmi import IAssociationsTable
from pyams_content.feature.navigation import IMenu, IMenuLinksContainer, IMenusContainer, \
    IMenusContainerTarget, Menu
from pyams_content.feature.navigation.interfaces import IMenuLinksContainerTarget
from pyams_content.feature.navigation.zmi.interfaces import IMenusTable
from pyams_content.reference.pictogram.zmi.widget import PictogramSelectFieldWidget
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.security import ProtectedViewObjectMixin
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_table.interfaces import IColumn
from pyams_utils.adapter import ContextRequestViewAdapter, NullAdapter, adapter_config
from pyams_utils.interfaces.intids import IUniqueID
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.helper.event import get_json_table_row_add_callback, \
    get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, IObjectHint, IObjectIcon, IObjectLabel, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle, IPropertiesEditForm
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager, IToolbarViewletManager
from pyams_zmi.table import TableElementEditor
from pyams_zmi.utils import get_object_hint, get_object_label

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='add-mailto-link.menu',
                context=IMenuLinksContainerTarget, layer=IAdminLayer, view=IAssociationsTable,
                manager=IContextAddingsViewletManager, weight=60)
class MenuLinksContainerMailtoLinkAddMenu(NullAdapter):
    """Menu links container mailto link add menu"""


@adapter_config(name='size',
                required=(IMenuLinksContainer, IAdminLayer, IAssociationsTable),
                provides=IColumn)
class MenuLinksContainerSizeColumn(NullAdapter):
    """Disabled size column in menu links container"""


#
# Menu add form
#

@viewlet_config(name='add-menu.menu',
                context=IMenusContainerTarget, layer=IAdminLayer, view=IAssociationsTable,
                manager=IToolbarViewletManager, weight=10)
class MenuAddAction(ProtectedViewObjectMixin, ContextAddAction):
    """Menu add action"""

    label = Menu.icon_hint
    href = 'add-menu.html'

    def get_href(self):
        """Menu URL target getter"""
        container = IMenusContainer(self.context)
        return absolute_url(container, self.request, self.href)


@ajax_form_config(name='add-menu.html',
                  context=IMenusContainer, layer=IPyAMSLayer)
class MenuAddForm(AdminModalAddForm):
    """Menu add form"""

    modal_class = 'modal-xl'

    subtitle = _("New navigation menu")
    legend = _("New menu properties")

    fields = Fields(IMenu).omit('__parent__', '__name__', 'visible')
    fields['pictogram_name'].widget_factory = PictogramSelectFieldWidget

    content_factory = IMenu

    def add(self, obj):
        oid = IUniqueID(obj).oid
        self.context[oid] = obj


@adapter_config(required=(IMenusContainer, IAdminLayer, MenuAddForm),
                provides=IFormTitle)
def menu_add_form_title(context, request, form):
    """Menu add form title getter"""
    parent = get_parent(context, IMenusContainerTarget)
    hint = get_object_hint(parent, request, form)
    label = get_object_label(parent, request, form)
    return TITLE_SPAN_BREAK.format(hint, label)


@adapter_config(required=(IMenusContainer, IAdminLayer, MenuAddForm),
                provides=IAJAXFormRenderer)
class MenuAddFormRenderer(ContextRequestViewAdapter):
    """Menu add form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        return {
            'status': 'success',
            'callbacks': [
                get_json_table_row_add_callback(self.context, self.request,
                                                IMenusTable, changes)
            ]
        }


@adapter_config(required=(IMenu, IAdminLayer, Interface),
                provides=IObjectIcon)
def menu_icon(context, request, view):
    """Menu icon getter"""
    return Menu.icon_class


@adapter_config(required=(IMenu, IAdminLayer, Interface),
                provides=IObjectHint)
def menu_hint(context, request, view):
    """Menu hint getter"""
    return request.localizer.translate(Menu.icon_hint)


@adapter_config(required=(IMenu, IAdminLayer, Interface),
                provides=IObjectLabel)
def menu_label(context, request, view):
    """Menu label getter"""
    return II18n(context).query_attribute('title', request=request)


@adapter_config(required=(IMenu, IAdminLayer, IMenusTable),
                provides=ITableElementEditor)
class MenuTableElementEditor(TableElementEditor):
    """Menu table element editor"""


@ajax_form_config(name='properties.html',
                  context=IMenu, layer=IPyAMSLayer)
@implementer(IPropertiesEditForm)
class MenuPropertiesEditForm(AdminModalEditForm):
    """Menu properties edit form"""

    @property
    def subtitle(self):
        return get_object_label(self.context, self.request, self)

    legend = _("Menu properties")

    fields = Fields(IMenu).omit('__parent__', '__name__', 'visible')
    fields['pictogram_name'].widget_factory = PictogramSelectFieldWidget


@adapter_config(required=(IMenu, IAdminLayer, IPropertiesEditForm),
                provides=IFormTitle)
def menu_edit_form_title(context, request, view):
    """Menu properties edit form title getter"""
    parent = get_parent(context, IMenusContainerTarget)
    hint = get_object_hint(parent, request, view)
    label = get_object_label(parent, request, view)
    return TITLE_SPAN_BREAK.format(hint, label)


@adapter_config(required=(IMenu, IAdminLayer, MenuPropertiesEditForm),
                provides=IAJAXFormRenderer)
class MenuEditFormRenderer(ContextRequestViewAdapter):
    """Menu edit form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        parent = get_parent(self.context, IMenusContainer)
        return {
            'status': 'success',
            'callbacks': [
                get_json_table_row_refresh_callback(parent, self.request,
                                                    IMenusTable, self.context)
            ]
        }



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from hypatia.interfaces import ICatalog
from hypatia.query import Any
from zope.intid.interfaces import IIntIds

from pyams_content.component.thesaurus import ICollectionsManager, ITagsManager
from pyams_content.feature.search.interfaces import ISearchFolder, ISearchFolderQuery, ISearchFormRequestParams
from pyams_content.shared.view.interfaces.query import IViewUserQuery
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_thesaurus.interfaces.thesaurus import IThesaurus
from pyams_utils.adapter import ContextAdapter, ContextRequestAdapter, adapter_config
from pyams_utils.registry import get_utility, query_utility


#
# Tags search adapters
#

@adapter_config(name='tags',
                required=ISearchFolderQuery,
                provides=IViewUserQuery)
class SearchFolderTagQuery(ContextAdapter):
    """Search folder tags query"""

    @staticmethod
    def get_user_params(request):
        tag = request.params.get('tag')
        if not tag:
            return
        manager = ITagsManager(request.root, None)
        if manager is None:
            return
        thesaurus = query_utility(IThesaurus, name=manager.thesaurus_name)
        if thesaurus is None:
            return
        term = thesaurus.terms.get(tag)
        if term is not None:
            catalog = get_utility(ICatalog)
            intids = query_utility(IIntIds)
            yield Any(catalog['tags'], (intids.queryId(term),))


@adapter_config(name='tags',
                required=(ISearchFolder, IPyAMSUserLayer),
                provides=ISearchFormRequestParams)
class SearchFormTagsRequestParams(ContextRequestAdapter):
    """Search form tags request params"""

    def get_params(self):
        """Request params getter"""
        tag = self.request.params.get('tag')
        if tag:
            yield {
                'name': 'tag',
                'value': tag
            }


#
# Themes search adapters
#

@adapter_config(name='themes',
                required=(ISearchFolder, IPyAMSUserLayer),
                provides=ISearchFormRequestParams)
class SearchFormThemesRequestParams(ContextRequestAdapter):
    """Search form themes request params"""

    def get_params(self):
        """Request params getter"""
        for theme in self.request.params.getall('theme'):
            yield {
                'name': 'theme',
                'value': theme
            }


#
# Collections search adapters
#

@adapter_config(name='collections',
                context=ISearchFolderQuery,
                provides=IViewUserQuery)
class SearchFolderCollectionQuery(ContextAdapter):
    """Search folder collections query"""

    @staticmethod
    def get_user_params(request):
        collection = request.params.get('collection')
        if not collection:
            return
        manager = ICollectionsManager(request.root, None)
        if manager is None:
            return
        thesaurus = query_utility(IThesaurus, name=manager.thesaurus_name)
        if thesaurus is None:
            return
        term = thesaurus.terms.get(collection)
        if term is not None:
            catalog = get_utility(ICatalog)
            intids = query_utility(IIntIds)
            yield Any(catalog['collections'], (intids.queryId(term),))


@adapter_config(name='collections',
                context=(ISearchFolder, IPyAMSUserLayer),
                provides=ISearchFormRequestParams)
class SearchFormCollectionsRequestParams(ContextRequestAdapter):
    """Search form tags request params"""

    def get_params(self):
        collection = self.request.params.get('collection')
        if collection:
            yield {
                'name': 'collection',
                'value': collection
            }



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.zmi.widget.seo module

This module define a custom widget which can be used to automatically display
an "SEO quality"
"""

__docformat__ = 'restructuredtext'

from zope.interface import implementer, implementer_only

from pyams_content.zmi.widget.interfaces import II18nSEOTextLineWidget, ISEOTextLineWidget
from pyams_form.browser.text import TextWidget
from pyams_form.interfaces import INPUT_MODE
from pyams_form.template import widget_template_config
from pyams_form.widget import FieldWidget
from pyams_i18n_views.widget import I18nWidget
from pyams_layer.interfaces import IPyAMSLayer


@widget_template_config(mode=INPUT_MODE,
                        template='templates/seo-textline-input.pt',
                        layer=IPyAMSLayer)
@implementer_only(ISEOTextLineWidget)
class SEOTextLineWidget(TextWidget):
    """SEO text line widget"""

    @property
    def length(self):
        """Get current length of text"""
        return len(self.value or '')

    @property
    def status(self):
        """Get widget status based on text length; a "good" length is between 40 and 66
        characters
        """
        status = 'success'
        length = self.length
        if length < 20 or length > 80:
            status = 'danger'
        elif length < 40 or length > 66:
            status = 'warning'
        return status


def SEOTextLineFieldWidget(field, request):  # pylint: disable=invalid-name
    """SEO text line field widget factory"""
    return FieldWidget(field, SEOTextLineWidget(request))


#
# I18n SEO text line widget
#

@implementer(II18nSEOTextLineWidget)
class I18nSEOTextLineWidget(I18nWidget):
    """I18n text line widget with SEO quality marker"""


def I18nSEOTextLineFieldWidget(field, request):
    """I18n text line field widget with SEO quality marker factory"""
    widget = I18nSEOTextLineWidget(request)
    widget.widget_factory = SEOTextLineFieldWidget
    return FieldWidget(field, widget)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.preview.zmi module

Management interface components used by preview feature.
"""

from pyramid.interfaces import IView

from pyams_content.feature.preview.interfaces import IPreviewTarget
from zope.interface import Interface

from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.actions import ContextAction, ContextActionsViewletManager
from pyams_template.template import layout_config
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalDisplayForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IToolbarViewletManager


__docformat__ = 'restructuredtext'

from pyams_content import _


@viewletmanager_config(name='pyams_context.preview',
                       context=IPreviewTarget, layer=IAdminLayer, view=IView,
                       manager=IToolbarViewletManager, weight=900)
class PreviewTargetActions(ContextActionsViewletManager):
    """Preview target actions"""


@viewlet_config(name='pyams_content.preview.modal',
                context=IPreviewTarget, layer=IAdminLayer, view=IView,
                manager=PreviewTargetActions, weight=10)
class ModalPreviewAction(ContextAction):
    """Modal preview action"""

    icon_class = 'fas fa-binoculars'
    label = _("Preview")

    href = 'modal-preview.html'
    modal_target = True


@pagelet_config(name='modal-preview.html',
                context=IPreviewTarget, layer=IPyAMSLayer)
@layout_config(template='templates/modal-preview.pt', layer=IPyAMSLayer)
class ModalPreview(AdminModalDisplayForm):
    """Modal preview form"""

    modal_class = 'modal-max'


@viewlet_config(name='pyams_content.preview.blank',
                context=IPreviewTarget, layer=IAdminLayer, view=Interface,
                manager=PreviewTargetActions, weight=20)
class BlankPreviewAction(ContextAction):
    """Blank window preview action"""

    icon_class = 'far fa-window-maximize'
    hint = _("Open preview in new tab")
    hint_placement = 'bottom'

    href = 'preview.html'
    target = '_blank'



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.cards.interfaces import CARDS_PARAGRAPH_ICON_CLASS, CARDS_PARAGRAPH_NAME, \
    CARDS_PARAGRAPH_RENDERERS, CARDS_PARAGRAPH_TYPE, ICardsParagraph
from pyams_content.component.paragraph import BaseParagraph, IBaseParagraph
from pyams_content.feature.renderer import RenderersVocabulary
from pyams_portal.portlets.cards import CardsContainer
from pyams_utils.factory import factory_config
from pyams_utils.vocabulary import vocabulary_config


@factory_config(ICardsParagraph)
@factory_config(IBaseParagraph, name=CARDS_PARAGRAPH_TYPE)
class CardsParagraph(CardsContainer, BaseParagraph):
    """Bootstrap cards paragraph"""

    factory_name = CARDS_PARAGRAPH_TYPE
    factory_label = CARDS_PARAGRAPH_NAME
    factory_intf = ICardsParagraph

    icon_class = CARDS_PARAGRAPH_ICON_CLASS

    renderer = FieldProperty(ICardsParagraph['renderer'])


@vocabulary_config(name=CARDS_PARAGRAPH_RENDERERS)
class CardsParagraphRenderersVocabulary(RenderersVocabulary):
    """Cards paragraph renderers vocabulary"""

    content_interface = ICardsParagraph



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.association.interfaces module

"""

from zope.annotation import IAttributeAnnotatable
from zope.container.constraints import containers, contains
from zope.container.interfaces import IOrderedContainer
from zope.interface import Attribute, Interface
from zope.schema import Bool, Choice
from pyams_content.component.paragraph.interfaces import IBaseParagraph, ParagraphRendererChoice


__docformat__ = 'restructuredtext'

from pyams_content import _


ASSOCIATION_CONTAINER_KEY = 'pyams_content.associations'
ASSOCIATION_VOCABULARY = 'pyams_content.associations'


class IAssociationItem(IAttributeAnnotatable):
    """Base association item interface"""

    containers('.IAssociationContainer')

    icon_class = Attribute("Icon class in associations list")
    icon_hint = Attribute("Icon hint in associations list")

    visible = Bool(title=_("Visible?"),
                   description=_("Is this item visible in front-office?"),
                   required=True,
                   default=True)

    def is_visible(self, request=None):
        """Is association item published?"""

    def get_url(self, request=None, view_name=None):
        """Get link URL"""


class IAssociationInfo(Interface):
    """Association information interface"""

    pictogram = Attribute("Association pictogram")

    user_title = Attribute("Association title proposed on public site")
    user_header = Attribute("Association header proposed on public site")
    user_icon = Attribute("Icon associated with user title")

    inner_title = Attribute("Inner content, if available")
    human_size = Attribute("Content size, if available")


class IAssociationContainer(IOrderedContainer):
    """Associations container interface"""

    contains(IAssociationItem)

    def append(self, value, notify=True):
        """Append given value to container"""

    def get_visible_items(self, request=None):
        """Get list of visible items"""


class IAssociationContainerTarget(IAttributeAnnotatable):
    """Associations container target interface"""


class IAssociationRenderer(Interface):
    """Association renderer adapter interface"""


#
# Associations paragraph
#

ASSOCIATION_PARAGRAPH_TYPE = 'associations'
ASSOCIATION_PARAGRAPH_NAME = _("Associations")
ASSOCIATION_PARAGRAPH_RENDERERS = 'PyAMS_content.paragraph.associations.renderers'
ASSOCIATION_PARAGRAPH_ICON_CLASS = 'fas fa-link'


class IAssociationParagraph(IBaseParagraph):
    """Associations paragraph interface"""

    renderer = ParagraphRendererChoice(description=_("Presentation template used for "
                                                     "associations"),
                                       renderers=ASSOCIATION_PARAGRAPH_RENDERERS)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration.skin.paragraph module

This module provides illustration paragraph rendering components.
"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.illustration.interfaces import IIllustrationParagraph
from pyams_content.component.illustration.skin.interfaces import \
    IIllustrationDefaultRendererSettings
from pyams_content.feature.renderer import DefaultContentRenderer, IContentRenderer
from pyams_portal.interfaces import DEFAULT_RENDERER_NAME
from pyams_layer.interfaces import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config


__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(IIllustrationDefaultRendererSettings)
class IllustrationParagraphDefaultRendererSettings(Persistent, Contained):
    """Illustration paragraph default renderer settings"""

    thumb_selection = FieldProperty(IIllustrationDefaultRendererSettings['thumb_selection'])


@adapter_config(name=DEFAULT_RENDERER_NAME,
                required=(IIllustrationParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/paragraph-default.pt', layer=IPyAMSLayer)
class IllustrationParagraphDefaultRenderer(DefaultContentRenderer):
    """Illustration paragraph default renderer"""

    label = _("Illustration (default)")

    settings_interface = IIllustrationDefaultRendererSettings



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.association.skin module

This module provides associations rendering components.
"""

from zope.interface import Interface

from pyams_content.component.association import IAssociationContainer
from pyams_content.component.association.interfaces import IAssociationInfo
from pyams_content.component.extfile import IExtFile
from pyams_content.component.links import IBaseLink
from pyams_layer.interfaces import IPyAMSLayer
from pyams_template.template import template_config
from pyams_viewlet.viewlet import ViewContentProvider, contentprovider_config


__docformat__ = 'restructuredtext'


class AssociationContainerRendererMixin:
    """Associations container renderer mixin"""

    attachments = None
    links = None

    def get_associations(self):
        """Associations getter"""
        yield from IAssociationContainer(self.context).get_visible_items(self.request)

    @staticmethod
    def get_link_info(item):
        """Association item information getter"""
        return IAssociationInfo(item)

    def update(self):
        super().update()
        self.attachments = []
        self.links = []
        for item in self.get_associations():
            if IExtFile.providedBy(item):
                self.attachments.append(item)
            elif IBaseLink.providedBy(item):
                self.links.append(item)


@contentprovider_config(name='pyams_content.associations',
                        layer=IPyAMSLayer, view=Interface)
@template_config(template='templates/association-viewlet.pt', layer=IPyAMSLayer)
class AssociationsViewlet(AssociationContainerRendererMixin, ViewContentProvider):
    """Associations viewlet"""



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery.zmi.file module

Gallery files management components.
"""

from pyramid.interfaces import IView
from zope.component import queryMultiAdapter
from zope.interface import Interface
from zope.lifecycleevent import ObjectCreatedEvent
from zope.schema._bootstrapinterfaces import WrongType

from pyams_content.component.gallery.interfaces import IGalleryContainer, IGalleryFile
from pyams_content.component.gallery.zmi import get_json_gallery_refresh_event
from pyams_content.component.gallery.zmi.helpers import get_json_gallery_media_refresh_event
from pyams_content.component.gallery.zmi.interfaces import IGalleryMediaThumbnailView, IGalleryMediasAddFields, \
    IGalleryMediasView
from pyams_content.component.paragraph.zmi import get_json_paragraph_toolbar_refresh_event
from pyams_file.file import get_magic_content_type
from pyams_file.interfaces.archive import IArchiveExtractor
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IGroup
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_security.permission import get_edit_permission
from pyams_security.security import ProtectedViewObjectMixin
from pyams_skin.interfaces.view import IModalPage
from pyams_skin.interfaces.viewlet import IContextActionsViewletManager
from pyams_skin.viewlet.actions import ContextAction, ContextAddAction
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config, query_adapter
from pyams_utils.factory import create_object, factory_config
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import ViewContentProvider, contentprovider_config, viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm, FormGroupSwitcher
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.viewlet import IToolbarViewletManager
from pyams_zmi.utils import get_object_label

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='add-media.menu',
                context=IGalleryContainer, layer=IAdminLayer,
                view=IGalleryMediasView,
                manager=IToolbarViewletManager, weight=10)
class GalleryMediaAddAction(ProtectedViewObjectMixin, ContextAddAction):
    """Gallery media add action"""

    label = _("Add media(s)")
    href = 'add-media.html'


@ajax_form_config(name='add-media.html',
                  context=IGalleryContainer, layer=IPyAMSLayer)
class GalleryMediaAddForm(AdminModalAddForm):
    """Gallery media add form"""

    subtitle = _("New medias")
    legend = _("New medias properties")

    fields = Fields(IGalleryMediasAddFields)
    content_factory = IGalleryFile

    def create_and_add(self, data):
        data = data.get(self, {})
        medias = []
        medias_data = data.pop('medias_data')
        filename = None
        if isinstance(medias_data, (list, tuple)):
            filename, medias_data = medias_data
        is_file = hasattr(medias_data, 'seek')
        if is_file:
            medias_data.seek(0)
        content_type = get_magic_content_type(medias_data)
        if isinstance(content_type, bytes):
            content_type = content_type.decode()
        if is_file:
            medias_data.seek(0)
        extractor = query_utility(IArchiveExtractor, name=content_type)
        if extractor is not None:
            contents = extractor.get_contents(medias_data)
        else:
            contents = ((medias_data, filename),)
        for content, filename in contents:
            try:
                media = self.create(data)
                self.add(media)
                media.data = filename, content
            except WrongType:
                continue
            else:
                medias.append(media)
        return medias

    def create(self, data):
        """Create new media from content factory"""
        media = create_object(self.content_factory)
        if media is not None:
            self.request.registry.notify(ObjectCreatedEvent(media))
            media.author = data.get('author')
        return media

    def add(self, obj):
        """Add new media to gallery"""
        self.context.append(obj)


@adapter_config(required=(IGalleryContainer, IAdminLayer, GalleryMediaAddForm),
                provides=IAJAXFormRenderer)
class GalleryMediaAddFormRenderer(ContextRequestViewAdapter):
    """Gallery media add form renderer"""

    def render(self, changes):
        """JSON result renderer"""
        if not changes:
            return None
        media = changes[0]
        result = {
            'status': 'success',
            'callbacks': [
                get_json_gallery_refresh_event(media, self.request, self.view)
            ]
        }
        event = get_json_paragraph_toolbar_refresh_event(media, self.request)
        if event is not None:
            result.setdefault('callbacks', []).append(event)
        return result


@contentprovider_config(name='gallery-media-thumbnail',
                        context=IGalleryFile, layer=IAdminLayer,
                        view=IGalleryMediasView)
@template_config(template='templates/gallery-thumbnail.pt', layer=IAdminLayer)
@factory_config(IGalleryMediaThumbnailView)
class GalleryMediaPreview(ViewContentProvider):
    """Gallery media preview"""

    def get_thumbnail_target(self):
        """Gallery media thumbnail target getter"""
        value = self.context.data
        if value is not None:
            view = queryMultiAdapter((value, self.request), IView, name='preview.html')
            if view is not None:
                return absolute_url(value, self.request, 'preview.html')
        return None


@viewlet_config(name='show-hide-media.action',
                context=IGalleryFile, layer=IAdminLayer, view=Interface,
                manager=IContextActionsViewletManager, weight=10,
                permission=VIEW_SYSTEM_PERMISSION)
class GalleryFileShowHideAction(ContextAction):
    """Gallery file show/hide action"""

    can_edit = False

    def __init__(self, context, request, view, manager):
        super().__init__(context, request, view, manager)
        gallery = get_parent(context, IGalleryContainer)
        if gallery is not None:
            edit_permission = get_edit_permission(request, context=gallery, view=view)
            self.can_edit = request.has_permission(edit_permission, context=context)

    @property
    def hint(self):
        """Hint getter"""
        if self.can_edit:
            return _("Show/hide media")
        return None

    css_class = 'btn-sm px-1'

    @property
    def icon_class(self):
        """Icon class getter"""
        if IGalleryFile(self.context).visible:
            icon_class = 'far fa-fw fa-eye'
        else:
            icon_class = 'far fa-fw fa-eye-slash text-danger'
        if self.can_edit:
            icon_class += ''
        return icon_class

    def get_href(self):
        """Icon URL getter"""
        return None

    @property
    def click_handler(self):
        """Action click handler getter"""
        if self.can_edit:
            return 'MyAMS.content.galleries.switchVisibleMedia'
        return None


#
# Gallery file properties
#

@adapter_config(required=(IGalleryFile, IPyAMSLayer, Interface),
                provides=IObjectLabel)
def gallery_file_label(context, request, view):
    """Gallery file label getter"""
    title = II18n(context).query_attribute('title', request=request)
    if not title:
        title = get_object_label(context.data, request, view)
    return title


@adapter_config(required=(IGalleryFile, IAdminLayer, IModalPage),
                provides=IFormTitle)
def gallery_file_modal_view_title(context, request, view):
    """Gallery file modal view title getter"""
    parent = get_parent(context, IGalleryContainer)
    return query_adapter(IFormTitle, request, parent, view)


@viewlet_config(name='show-properties.action',
                context=IGalleryFile, layer=IAdminLayer, view=Interface,
                manager=IContextActionsViewletManager, weight=20,
                permission=VIEW_SYSTEM_PERMISSION)
class GalleryFilePropertiesAction(ContextAction):
    """Gallery file properties action"""

    hint = _("Media properties")
    css_class = 'btn-sm px-1'
    icon_class = 'far fa-edit'

    href = 'properties.html'
    modal_target = True


@ajax_form_config(name='properties.html',
                  context=IGalleryFile, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class GalleryFilePropertiesEditForm(AdminModalEditForm):
    """Gallery file properties edit form"""

    @property
    def subtitle(self):
        translate = self.request.localizer.translate
        return translate(_("Media: {}")).format(
            get_object_label(self.context, self.request, self))

    legend = _("Gallery media properties")
    modal_class = 'modal-xl'

    fields = Fields(IGalleryFile).select('data', 'title', 'alt_title', 'description', 'author')

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        data = self.widgets.get('data')
        if data is not None:
            data.required = False


@adapter_config(name='gallery-media-sound',
                required=(IGalleryFile, IAdminLayer, GalleryFilePropertiesEditForm),
                provides=IGroup)
class GalleryFilePropertiesEditFormSoundGroup(FormGroupSwitcher):
    """Gallery file properties edit form sound group"""

    legend = _("Audio content")

    fields = Fields(IGalleryFile).select('sound', 'sound_title', 'sound_description')


@adapter_config(required=(IGalleryFile, IAdminLayer, GalleryFilePropertiesEditForm),
                provides=IAJAXFormRenderer)
class GalleryFilePropertiesEditFormRenderer(ContextRequestViewAdapter):
    """Gallery file properties edit form renderer"""

    def render(self, changes):
        if not changes:
            return None
        return {
            'status': 'success',
            'message': self.request.localizer.translate(self.view.success_message),
            'callbacks': [
                get_json_gallery_media_refresh_event(self.context, self.request, self.view)
            ]
        }


#
# Gallery file remover
#

@viewlet_config(name='delete-media.action',
                context=IGalleryFile, layer=IAdminLayer, view=Interface,
                manager=IContextActionsViewletManager, weight=99,
                permission=VIEW_SYSTEM_PERMISSION)
class GalleryFileDeleteAction(ContextAction):
    """Gallery file delete action"""

    def __new__(cls, context, request, view, manager):
        gallery = get_parent(context, IGalleryContainer)
        if gallery is not None:
            edit_permission = get_edit_permission(request, context=gallery, view=view)
            if not request.has_permission(edit_permission, context=context):
                return None
        return ContextAction.__new__(cls)

    hint = _("Delete media")
    css_class = 'btn-sm px-1'
    icon_class = 'fas fa-trash'

    def get_href(self):
        """Icon URL getter"""
        return None

    click_handler = 'MyAMS.content.galleries.removeMedia'



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.fields.handler.interfaces module

This module defines custom form handlers interfaces.
"""

from zope.interface import Interface
from zope.schema import TextLine

from pyams_fields.interfaces import IFormHandlerInfo
from pyams_i18n.schema import I18nHTMLField, I18nTextLineField
from pyams_utils.schema import MailAddressField


__docformat__ = 'restructuredtext'

from pyams_content import _


MAILTO_HANDLER_ANNOTATION_KEY = 'pyams_content.handler::mailto'


class IMailtoHandlerInfo(IFormHandlerInfo):
    """Simple mailto handler info interface"""

    service_name = I18nTextLineField(title=_("Service name"),
                                     description=_("This name will be used as subject in all email "
                                                   "messages sent by this contact form"),
                                     required=False)

    source_name = TextLine(title=_("Source name"),
                           description=_("Name of mail data sender"),
                           required=False)

    source_address = MailAddressField(title=_("Source address"),
                                      description=_("Mail address from which form data is sent"),
                                      required=True)

    target_name = TextLine(title=_("Recipient name"),
                           description=_("Name of data recipient"),
                           required=False)

    target_address = MailAddressField(title=_("Recipient address"),
                                      description=_("Mail address to which form data is sent"),
                                      required=True)

    notification_message = I18nHTMLField(title=_("Notification message"),
                                         description=_("This message will be sent to target address defined just "
                                                       "before; you can include form fields values by including "
                                                       "their name, enclosed in {brackets}, or {_reference} to get "
                                                       "the submission ID; if you don't provide any template, message "
                                                       "will be filled only with the raw data entered into this form"),
                                         required=False)

    confirm_message = I18nHTMLField(title=_("Confirmation message"),
                                    description=_("If an email address is provided in the form, this message will be "
                                                  "returned to the contact; you can include all form fields values by "
                                                  "including their name, enclosed in {brackets}, or {_reference} to "
                                                  "include submission ID"),
                                    required=False)


class IMailtoHandlerTarget(Interface):
    """Simple mailto handler target interface"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface

from pyams_content.reference import IReferenceManager, IReferenceTable
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_skin.interfaces.viewlet import IBreadcrumbItem
from pyams_skin.viewlet.breadcrumb import BreadcrumbItem
from pyams_utils.adapter import adapter_config
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel


__docformat__ = 'restructuredtext'

from pyams_content import _


REFERENCE_MANAGER_LABEL = _("References")


@adapter_config(required=(IReferenceManager, IPyAMSLayer, Interface),
                provides=IObjectLabel)
def reference_manager_label(context, request, view):
    """Reference manager label"""
    return request.localizer.translate(REFERENCE_MANAGER_LABEL)


@adapter_config(required=(IReferenceManager, IAdminLayer, Interface),
                provides=IBreadcrumbItem)
class ReferenceManagerBreadcrumbAdapter(BreadcrumbItem):
    """References tables manager breadcrumb adapter"""

    label = _("References tables")
    view_name = None


@adapter_config(required=(IReferenceTable, IAdminLayer, Interface),
                provides=IBreadcrumbItem)
class ReferenceTableBreadcrumbs(BreadcrumbItem):
    """Reference table breadcrumb item"""

    @property
    def label(self):
        return II18n(self.context).query_attribute('short_name', request=self.request)

    view_name = 'admin'



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.portlet.title module

This module defines a small shared content title portlet, which can be used
instead of header portlet just to render content title.
"""

from zope.schema.fieldproperty import FieldProperty

from pyams_content.shared.common.portlet.interfaces import ISharedContentTitlePortletSettings, TITLE_PORTLET_NAME
from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
from pyams_utils.factory import factory_config

__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(provided=ISharedContentTitlePortletSettings)
class SharedContentTitlePortletSettings(PortletSettings):
    """Shared content title portlet settings"""

    display_publication_date = FieldProperty(ISharedContentTitlePortletSettings['display_publication_date'])
    publication_date_prefix = FieldProperty(ISharedContentTitlePortletSettings['publication_date_prefix'])
    display_specificities = FieldProperty(ISharedContentTitlePortletSettings['display_specificities'])


@portlet_config(permission=None)
class SharedContentTitlePortlet(Portlet):
    """Shared content title portlet"""

    name = TITLE_PORTLET_NAME
    label = _("Content title")

    settings_factory = ISharedContentTitlePortletSettings
    toolbar_css_class = 'fas fa-bold'



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.manager module

This module defines views manager shared tool.
"""

from pyramid.events import subscriber
from zope.component.interfaces import ISite
from zope.lifecycleevent.interfaces import IObjectAddedEvent

from pyams_content.shared.common.manager import SharedTool
from pyams_content.shared.view.interfaces import IViewManager, VIEW_CONTENT_TYPE
from pyams_utils.factory import factory_config
from pyams_utils.traversing import get_parent

__docformat__ = 'restructuredtext'


@factory_config(IViewManager)
class ViewManager(SharedTool):
    """View manager class"""

    shared_content_type = VIEW_CONTENT_TYPE
    shared_content_menu = False


@subscriber(IObjectAddedEvent, context_selector=IViewManager)
def handle_added_view_manager(event):
    """Register view manager when added"""
    site = get_parent(event.newParent, ISite)
    registry = site.getSiteManager()
    if registry is not None:
        registry.registerUtility(event.object, IViewManager)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi.types.container module

This module defines components which are used for shared contents types management interface.
"""

from pyramid.view import view_config

from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_content.shared.common.interfaces.types import ITypedDataManager, ITypedSharedTool
from pyams_content.shared.common.zmi.types.interfaces import ISharedToolTypesTable
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.helper.container import delete_container_element, switch_element_attribute
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.table import NameColumn, ReorderColumn, SortableTable, TableAdminView, TrashColumn, \
    VisibilityColumn
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='data-types.menu',
                context=ITypedSharedTool, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=405,
                permission=MANAGE_TOOL_PERMISSION)
class SharedToolTypesMenu(NavigationMenuItem):
    """Shared tool data types menu"""

    label = _("Content types")
    href = '#data-types.html'


@factory_config(ISharedToolTypesTable)
class SharedToolTypesTable(SortableTable):
    """Shared tool data types table"""

    container_class = ITypedDataManager

    display_if_empty = True


@adapter_config(required=(ITypedSharedTool, IAdminLayer, ISharedToolTypesTable),
                provides=IValues)
class SharedToolTypesTableValues(ContextRequestViewAdapter):
    """Shared tool data types table values"""

    @property
    def values(self):
        """Data types values getter"""
        yield from ITypedDataManager(self.context).values()


@adapter_config(name='reorder',
                required=(ITypedSharedTool, IAdminLayer, ISharedToolTypesTable),
                provides=IColumn)
class SharedToolTypesReorderColumn(ReorderColumn):
    """Shared tool data types table reorder column"""


@view_config(name='reorder.json',
             context=ITypedDataManager, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=MANAGE_TOOL_PERMISSION)
def reorder_types_table(request):
    """Reorder shared tool data types"""
    order = request.params.get('order').split(';')
    request.context.updateOrder(order)
    return {
        'status': 'success'
    }


@adapter_config(name='visible',
                required=(ITypedSharedTool, IAdminLayer, ISharedToolTypesTable),
                provides=IColumn)
class SharedToolTypesVisibleColumn(VisibilityColumn):
    """Shared tool data types table visible column"""

    hint = _("Click icon to enable or disable content type")


@view_config(name='switch-visible-item.json',
             context=ITypedDataManager, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def switch_visible_item(request):
    """Switch visible item"""
    return switch_element_attribute(request)


@adapter_config(name='label',
                required=(ITypedSharedTool, IAdminLayer, ISharedToolTypesTable),
                provides=IColumn)
class SharedToolTypesLabelColumn(NameColumn):
    """Shared tool data types table label column"""

    i18n_header = _("Label")


@adapter_config(name='trash',
                required=(ITypedSharedTool, IAdminLayer, ISharedToolTypesTable),
                provides=IColumn)
class SharedToolTypesTrashColumn(TrashColumn):
    """Shared tool data types table trash column"""


@view_config(name='delete-element.json',
             context=ITypedDataManager, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=MANAGE_TOOL_PERMISSION)
def delete_data_type(request):
    """Delete data type"""
    return delete_container_element(request)


@pagelet_config(name='data-types.html',
                context=ITypedSharedTool, layer=IPyAMSLayer,
                permission=MANAGE_TOOL_PERMISSION)
class SharedToolTypesView(TableAdminView):
    """Shared tool data types view"""

    title = _("Content types")

    table_class = ISharedToolTypesTable
    table_label = _("Shared tool content types list")



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.thesaurus.interfaces module

This module is used to assign tags, themes and collections to contents.
These elements are based on thesaurus contents.
"""

from zope.interface import Attribute, Interface, Invalid, invariant
from zope.schema import Bool, Choice

from pyams_thesaurus.interfaces import THESAURUS_NAMES_VOCABULARY
from pyams_thesaurus.interfaces.thesaurus import IThesaurusContextManager, \
    IThesaurusContextManagerTarget
from pyams_thesaurus.schema import ThesaurusTermsListField
from pyams_utils.interfaces.inherit import IInheritInfo

__docformat__ = 'restructuredtext'

from pyams_content import _


#
# Tags management
#

TAGS_MANAGER_KEY = 'pyams_content.tags.manager'
TAGS_INFO_KEY = 'pyams_content.tags.info'


class ITagsManager(IThesaurusContextManager):
    """Tags manager interface"""

    enable_glossary = Bool(title=_("Enable glossary"),
                           required=True,
                           default=False)

    glossary_thesaurus_name = Choice(title=_("Glossary thesaurus name"),
                                     vocabulary=THESAURUS_NAMES_VOCABULARY,
                                     required=False)

    glossary = Attribute("Glossary getter")

    @invariant
    def check_glossary_thesaurus(self):
        if self.enable_glossary and not self.glossary_thesaurus_name:
            raise Invalid(_("You must specify a glossary thesaurus to activate it!"))


class ITagsManagerTarget(IThesaurusContextManagerTarget):
    """Marker interface for tags manager"""


class ITagsInfo(Interface):
    """Tags information interface"""

    tags = ThesaurusTermsListField(title=_("Tags"),
                                   required=False)


class ITagsTarget(Interface):
    """Tags target interface"""


#
# Themes management
#

THEMES_MANAGER_KEY = 'pyams_content.themes.manager'
THEMES_INFO_KEY = 'pyams_content.themes.info'


class IThemesManager(IThesaurusContextManager):
    """Themes manager interface"""


class IThemesManagerTarget(IThesaurusContextManagerTarget):
    """Marker interface for tools managing themes"""


class IThemesInfo(IInheritInfo):
    """Themes information interface"""

    themes = ThesaurusTermsListField(title=_("Themes"),
                                     required=False)


class IThemesTarget(Interface):
    """Themes target interface"""


#
# Collections management
#

COLLECTIONS_MANAGER_KEY = 'pyams_content.collections.manager'
COLLECTIONS_INFO_KEY = 'pyams_content.collections.info'


class ICollectionsManager(IThesaurusContextManager):
    """Collections manager interface"""


class ICollectionsManagerTarget(IThesaurusContextManagerTarget):
    """Marker interface for tools managing collections"""


class ICollectionsInfo(Interface):
    """Collections information interface"""

    collections = ThesaurusTermsListField(title=_("Collections"),
                                          required=False)


class ICollectionsTarget(Interface):
    """Collections target interface"""



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.sitemap module

This module defines main sitemap management components.
"""

__docformat__ = 'restructuredtext'

from datetime import datetime
from itertools import product

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import Any, Eq
from pyramid.view import view_config
from zope.intid import IIntIds
from zope.schema.vocabulary import getVocabularyRegistry

from pyams_catalog.query import CatalogResultSet
from pyams_content.feature.sitemap.interfaces import ISitemapExtension
from pyams_content.root import ISiteRootToolsConfiguration
from pyams_content.shared.common import IBaseSharedTool, SHARED_CONTENT_TYPES_VOCABULARY
from pyams_i18n.interfaces import II18nManager
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_site.interfaces import ISiteRoot
from pyams_utils.list import unique_iter
from pyams_utils.registry import get_all_utilities_registered_for, get_utility
from pyams_utils.timezone import tztime
from pyams_workflow.interfaces import IWorkflow, IWorkflowPublicationInfo


@view_config(name='robots.txt',
             context=ISiteRoot, request_type=IPyAMSUserLayer,
             renderer='templates/robots.pt')
def site_root_robots_view(request):
    """Site root robots.txt view"""
    request.response.content_type = 'text/plain'
    return {
        'tools_configuration': ISiteRootToolsConfiguration(request.root),
        'disallow': [
            tool
            for tool in get_all_utilities_registered_for(IBaseSharedTool)
            if not tool.shared_content_menu
        ]
    }


@view_config(name='humans.txt',
             context=ISiteRoot, request_type=IPyAMSUserLayer,
             renderer='templates/humans.pt')
def site_root_humans_view(request):
    """Site root humans.txt view"""
    request.response.content_type = 'text/plain'
    return {}


@view_config(name='sitemap.xml',
             context=ISiteRoot, request_type=IPyAMSUserLayer,
             renderer='templates/root-sitemap.pt')
class SiteRootSitemapView:
    """Site root sitemap view"""

    def __init__(self, request):
        self.request = request

    def __call__(self):
        self.request.response.content_type = 'text/xml'
        return {}

    @property
    def sources(self):
        """Sitemap sources"""
        timestamp = tztime(datetime.utcnow()).isoformat()
        for tool in get_all_utilities_registered_for(IBaseSharedTool):
            if not tool.shared_content_menu:
                continue
            publication_info = IWorkflowPublicationInfo(tool, None)
            if (publication_info is None) or publication_info.is_visible(self.request):
                yield timestamp, tool
        for name, adapter in self.request.registry.getAdapters(
                (self.request.context, self.request),
                ISitemapExtension):
            source = adapter.source
            if source is not None:
                yield timestamp, source


@view_config(name='sitemap.xml',
             context=IBaseSharedTool, request_type=IPyAMSUserLayer,
             renderer='templates/tool-sitemap.pt')
class SharedToolSitemapView:
    """Shared tool sitemap view"""

    def __init__(self, request):
        self.request = request

    def __call__(self):
        self.request.response.content_type = 'text/xml'
        return {}

    @property
    def contents(self):
        """Sitemap contents getter"""
        context = self.request.context
        catalog = get_utility(ICatalog)
        intids = get_utility(IIntIds)
        workflow = IWorkflow(context)
        vocabulary = getVocabularyRegistry().get(context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = Eq(catalog['parents'], intids.register(context)) & \
            Any(catalog['content_type'], vocabulary.by_value.keys()) & \
            Any(catalog['workflow_state'], workflow.visible_states)
        for version in unique_iter(CatalogResultSet(CatalogQuery(catalog).query(params))):
            yield from product(II18nManager(version).get_languages(), (version,))



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.portal module

"""

from pyams_content.shared.common import IBaseSharedTool
from pyams_content.shared.common.interfaces import IBaseContentPortalContext, \
    ISharedContentPortalPage
from pyams_portal.interfaces import IPortalPage, PORTAL_PAGE_KEY
from pyams_portal.page import PortalPage
from pyams_portal.utils import get_portal_page
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.traversing import get_parent
from pyams_utils.zodb import volatile_property

__docformat__ = 'restructuredtext'


class SharedContentPortalPageMixin:
    """Shared content portal page mixin class"""

    @volatile_property
    def can_inherit(self):
        page = get_portal_page(self.parent, page_name=self.name)
        return page.template is not None

    @property
    def parent(self):
        return get_parent(self, IBaseSharedTool, allow_context=False)


#
# SHared content portal page
#

@factory_config(ISharedContentPortalPage)
class SharedContentPortalPage(SharedContentPortalPageMixin, PortalPage):
    """Shared content portal page"""


@adapter_config(required=IBaseContentPortalContext,
                provides=IPortalPage)
def shared_content_portal_page(context, page_name=''):
    """Shared content portal page adapter"""

    def set_page_name(page):
        """Set page name after creation"""
        page.name = page_name

    key = f'{PORTAL_PAGE_KEY}::{page_name}' if page_name else PORTAL_PAGE_KEY
    return get_annotation_adapter(context, key, ISharedContentPortalPage,
                                  name=f'++page++{page_name}',
                                  callback=set_page_name)


@adapter_config(name='header',
                required=IBaseContentPortalContext,
                provides=IPortalPage)
def shared_content_portal_page_header(context):
    """Shared content portal page header adapter"""
    return shared_content_portal_page(context, page_name='header')


@adapter_config(name='footer',
                required=IBaseContentPortalContext,
                provides=IPortalPage)
def shared_content_portal_page_footer(context):
    """Shared content portal page footer adapter"""
    return shared_content_portal_page(context, page_name='footer')



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.reference.interfaces module

"""

from zope.container.constraints import containers, contains
from zope.container.interfaces import IContainer
from zope.interface import Interface

from pyams_content.interfaces import IBaseContent, REFERENCE_MANAGER_ROLE
from pyams_security.schema import PrincipalsSetField

__docformat__ = 'restructuredtext'

from pyams_content import _


class IReferenceInfo(IBaseContent):
    """Base reference interface"""

    containers('.IReferenceTable')


class IReferenceTable(IBaseContent):
    """Reference table interface"""

    containers('.IReferenceTableContainer')
    contains(IReferenceInfo)


REFERENCE_TABLE_ROLES = 'pyams_content.reference.roles'


class IReferenceTableRoles(Interface):
    """Reference table roles interface"""

    managers = PrincipalsSetField(title=_("Table managers"),
                                  description=_("Table managers can handle all table contents"),
                                  role_id=REFERENCE_MANAGER_ROLE,
                                  required=False)


class IReferenceManager(IBaseContent, IContainer):
    """References tables container"""

    contains(IReferenceTable)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.video.provider.interfaces module

This module defines settings interfaces of external videos providers.
"""

from zope.schema import Bool, Choice, Int, Text, TextLine

from pyams_content.component.video.interfaces import IExternalVideoSettings
from pyams_utils.schema import ColorField

__docformat__ = 'restructuredtext'

from pyams_content import _


class ICustomVideoSettings(IExternalVideoSettings):
    """Custom external video provider settings"""

    integration_code = Text(title=_("Integration code"),
                            description=_("Please select integration code provided by your video provider, "
                                          "and paste it here"),
                            required=True)


class IYoutubeVideoSettings(IExternalVideoSettings):
    """Youtube video provider settings"""

    video_id = TextLine(title=_("Video ID"),
                        description=_("To get video ID, just use the 'Share' button in Youtube platform and "
                                      "copy/paste the given URL here"),
                        required=True)

    start_at = TextLine(title=_("Start at"),
                        description=_("Position at which to start video, in 'seconds' or 'minutes:seconds' format"),
                        required=False,
                        default='0:00')

    stop_at = TextLine(title=_("Stop at"),
                       description=_("Position at which to stop video, in 'seconds' or 'minutes:seconds' format"),
                       required=False)

    autoplay = Bool(title=_("Auto play?"),
                    description=_("If 'yes', video is started automatically on page load"),
                    required=True,
                    default=False)

    loop = Bool(title=_("Loop playback?"),
                description=_("If 'yes', video is played indefinitely"),
                required=True,
                default=False)

    show_commands = Bool(title=_("Show commands?"),
                         description=_("Show video player commands"),
                         required=True,
                         default=True)

    hide_branding = Bool(title=_("Hide branding?"),
                         description=_("If 'no', Youtube branding will be displayed"),
                         required=True,
                         default=True)

    show_related = Bool(title=_("Show related videos?"),
                        description=_("Show related videos when video end"),
                        required=True,
                        default=False)

    allow_fullscreen = Bool(title=_("Allow full screen?"),
                            description=_("If 'yes', video can be displayed in full screen"),
                            required=True,
                            default=True)

    disable_keyboard = Bool(title=_("Disable keyboard?"),
                            description=_("If 'yes', video player can't be controlled via keyboard shortcuts"),
                            required=True,
                            default=False)

    width = Int(title=_("Video width"),
                description=_("Initial video frame width; mandatory for old browsers but may be overridden by "
                              "presentation skin"),
                required=True,
                min=200,
                default=720)

    height = Int(title=_("Video height"),
                 description=_("Initial video frame height; mandatory for old browsers but may be overridden by "
                               "presentation skin"),
                 required=True,
                 min=200,
                 default=405)


class IDailymotionVideoSettings(IExternalVideoSettings):
    """Dailymotion video provider settings"""

    video_id = TextLine(title=_("Video ID"),
                        description=_("To get video ID, just use the 'Share' button in Dailymotion platform, "
                                      "click on \"Copy link\" and paste the given URL here"),
                        required=True)

    start_at = TextLine(title=_("Start at"),
                        description=_("Position at which to start video, in 'seconds' or 'minutes:seconds' format"),
                        required=False,
                        default='0:00')

    autoplay = Bool(title=_("Auto play?"),
                    description=_("If 'yes', video is started automatically on page load"),
                    required=True,
                    default=False)

    show_info = Bool(title=_("Show video info?"),
                     description=_("If 'no', video title and information won't be displayed"),
                     required=True,
                     default=True)

    show_commands = Bool(title=_("Show commands?"),
                         description=_("Show video player commands"),
                         required=True,
                         default=True)

    ui_theme = Choice(title=_("UI theme"),
                      description=_("Default base color theme"),
                      values=('dark', 'light'),
                      default='dark')

    show_branding = Bool(title=_("Show branding?"),
                         description=_("If 'yes', Dailymotion branding will be displayed"),
                         required=True,
                         default=False)

    show_endscreen = Bool(title=_("Show end screen?"),
                          description=_("Show end screen when video end"),
                          required=True,
                          default=False)

    allow_fullscreen = Bool(title=_("Allow full screen?"),
                            description=_("If 'yes', video can be displayed in full screen"),
                            required=True,
                            default=True)

    allow_sharing = Bool(title=_("Allow sharing?"),
                         description=_("If 'no', video sharing will be disabled"),
                         required=True,
                         default=True)

    width = Int(title=_("Video width"),
                description=_("Initial video frame width; mandatory for old browsers but may be overridden by "
                              "presentation skin"),
                required=True,
                min=200,
                default=720)

    height = Int(title=_("Video height"),
                 description=_("Initial video frame height; mandatory for old browsers but may be overridden by "
                               "presentation skin"),
                 required=True,
                 min=200,
                 default=405)


class IVimeoVideoSettings(IExternalVideoSettings):
    """Vimeo video provider settings"""

    video_id = TextLine(title=_("Video ID"),
                        description=_("To get video ID, just use the 'Share' button in Vimeo platform, "
                                      "click on \"Link\" entry and copy/paste the given URL here"),
                        required=True)

    show_title = Bool(title=_("Show title?"),
                      description=_("If 'no', video title won't be displayed"),
                      required=True,
                      default=True)

    show_signature = Bool(title=_("Show signature?"),
                          description=_("If 'no', video's author signature won't be displayed"),
                          required=True,
                          default=True)

    color = ColorField(title=_("Infos color"),
                       description=_("Color used for title and signature"),
                       required=True,
                       default='ffffff')

    autoplay = Bool(title=_("Auto play?"),
                    description=_("If 'yes', video is started automatically on page load"),
                    required=True,
                    default=False)

    loop = Bool(title=_("Loop playback?"),
                description=_("If 'yes', video is played indefinitely"),
                required=True,
                default=False)

    allow_fullscreen = Bool(title=_("Allow full screen?"),
                            description=_("If 'yes', video can be displayed in full screen"),
                            required=True,
                            default=True)

    width = Int(title=_("Video width"),
                description=_("Initial video frame width; mandatory for old browsers but may be overridden by "
                              "presentation skin"),
                required=True,
                min=200,
                default=720)

    height = Int(title=_("Video height"),
                 description=_("Initial video frame height; mandatory for old browsers but may be overridden by "
                               "presentation skin"),
                 required=True,
                 min=200,
                 default=405)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

import logging
from importlib import import_module

from zope.dublincore.interfaces import IZopeDublinCore

from pyams_catalog.generations import check_required_indexes
from pyams_catalog.i18n import I18nTextIndexWithInterface
from pyams_catalog.index import DatetimeIndexWithInterface, FieldIndexWithInterface, \
    KeywordIndexWithInterface
from pyams_catalog.interfaces import DATE_RESOLUTION, MINUTE_RESOLUTION
from pyams_catalog.nltk import get_fulltext_lexicon
from pyams_content.component.thesaurus import ICollectionsInfo, ITagsInfo, IThemesInfo
from pyams_content.interfaces import CONTRIBUTOR_ROLE, IBaseContent, IObjectTypes, MANAGER_ROLE, OWNER_ROLE, \
    PILOT_ROLE, WEBMASTER_ROLE
from pyams_content.reference.pictogram import IPictogramTable
from pyams_content.root import ISiteRootToolsConfiguration
from pyams_content.shared.alert.interfaces import IAlertManager
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_content.shared.common.interfaces.types import IWfTypedSharedContent
from pyams_content.shared.form.interfaces import IFormManager
from pyams_content.shared.topic.interfaces import ITopicManager
from pyams_content.shared.view.interfaces import IViewManager
from pyams_security.index import PrincipalsRoleIndex
from pyams_site.generations import check_required_utilities
from pyams_site.interfaces import ISiteGenerations
from pyams_thesaurus.index import ThesaurusTermsListFieldIndex
from pyams_utils.interfaces.traversing import IPathElements
from pyams_utils.registry import get_pyramid_registry, utility_config
from pyams_workflow.interfaces import IWorkflowPublicationInfo, IWorkflowState


LOGGER = logging.getLogger('PyAMS (content)')

REQUIRED_UTILITIES = ()

REQUIRED_TABLES = (
    (IPictogramTable, 'pictograms'),
)

REQUIRED_TOOLS = (
    (IViewManager, 'views'),
    (IAlertManager, 'alerts'),
    (IFormManager, 'forms'),
    (ITopicManager, 'topics')
)

REQUIRED_INDEXES = [
    ('object_types', KeywordIndexWithInterface, {
        'interface': IObjectTypes,
        'discriminator': 'object_types'
    }),
    ('content_type', FieldIndexWithInterface, {
        'interface': IBaseContent,
        'discriminator': 'content_type'
    }),
    ('data_type', FieldIndexWithInterface, {
        'interface': IWfTypedSharedContent,
        'discriminator': 'data_type'
    }),
    ('role:owner', PrincipalsRoleIndex, {
        'role_id': OWNER_ROLE
    }),
    ('role:pilot', PrincipalsRoleIndex, {
        'role_id': PILOT_ROLE
    }),
    ('role:manager', PrincipalsRoleIndex, {
        'role_id': MANAGER_ROLE
    }),
    ('role:contributor', PrincipalsRoleIndex, {
        'role_id': CONTRIBUTOR_ROLE
    }),
    ('role:webmaster', PrincipalsRoleIndex, {
        'role_id': WEBMASTER_ROLE
    }),
    ('parents', KeywordIndexWithInterface, {
        'interface': IPathElements,
        'discriminator': 'parents'
    }),
    ('workflow_state', FieldIndexWithInterface, {
        'interface': IWorkflowState,
        'discriminator': 'state'
    }),
    ('workflow_principal', FieldIndexWithInterface, {
        'interface': IWorkflowState,
        'discriminator': 'state_principal'
    }),
    ('modifiers', KeywordIndexWithInterface, {
        'interface': IWfSharedContent,
        'discriminator': 'modifiers'
    }),
    ('created_date', DatetimeIndexWithInterface, {
        'interface': IZopeDublinCore,
        'discriminator': 'created',
        'resolution': DATE_RESOLUTION
    }),
    ('modified_date', DatetimeIndexWithInterface, {
        'interface': IZopeDublinCore,
        'discriminator': 'modified',
        'resolution': DATE_RESOLUTION
    }),
    ('publication_date', DatetimeIndexWithInterface, {
        'interface': IWorkflowPublicationInfo,
        'discriminator': 'publication_date',
        'resolution': MINUTE_RESOLUTION
    }),
    ('effective_date', DatetimeIndexWithInterface, {
        'interface': IWorkflowPublicationInfo,
        'discriminator': 'publication_effective_date',
        'resolution': MINUTE_RESOLUTION
    }),
    ('push_end_date', DatetimeIndexWithInterface, {
        'interface': IWorkflowPublicationInfo,
        'discriminator': 'push_end_date_index',
        'resolution': MINUTE_RESOLUTION
    }),
    ('expiration_date', DatetimeIndexWithInterface, {
        'interface': IWorkflowPublicationInfo,
        'discriminator': 'publication_expiration_date',
        'resolution': MINUTE_RESOLUTION
    }),
    ('first_publication_date', DatetimeIndexWithInterface, {
        'interface': IWorkflowPublicationInfo,
        'discriminator': 'first_publication_date',
        'resolution': MINUTE_RESOLUTION
    }),
    ('content_publication_date', DatetimeIndexWithInterface, {
        'interface': IWorkflowPublicationInfo,
        'discriminator': 'content_publication_date',
        'resolution': MINUTE_RESOLUTION
    }),
    ('visible_publication_date', DatetimeIndexWithInterface, {
        'interface': IWorkflowPublicationInfo,
        'discriminator': 'visible_publication_date',
        'resolution': MINUTE_RESOLUTION
    }),
    ('tags', ThesaurusTermsListFieldIndex, {
        'interface': ITagsInfo,
        'discriminator': 'tags',
        'include_parents': False,
        'include_synonyms': False
    }),
    ('themes', ThesaurusTermsListFieldIndex, {
        'interface': IThemesInfo,
        'discriminator': 'themes',
        'include_parents': False,
        'include_synonyms': False
    }),
    ('themes_tree', ThesaurusTermsListFieldIndex, {
        'interface': IThemesInfo,
        'discriminator': 'themes',
        'include_parents': True,
        'include_synonyms': False
    }),
    ('themes_all', ThesaurusTermsListFieldIndex, {
        'interface': IThemesInfo,
        'discriminator': 'themes',
        'include_parents': True,
        'include_synonyms': True
    }),
    ('collections', ThesaurusTermsListFieldIndex, {
        'interface': ICollectionsInfo,
        'discriminator': 'collections',
        'include_parents': False,
        'include_synonyms': False
    })
]


#
# Checker for required shared tables
#

def check_required_tables(site, tables=REQUIRED_TABLES, registry=None):
    """Check for required reference tables

    :param site: site root to check for tables
    :param tables: iterable of reference tables to check or create; each occurrence is a tuple
        of three elements containing configuration attribute name, default table name and table
        factory
    :param registry: optional registry object

    For each table which may be registered, the check if it's already defined into current site
    configuration; if not, a factory lookup is done: a custom factory can be defined using a
    setting named *pyams_content.config.{name}_factory*, where name is the default tool name;
    otherwise, a default object factory lookup for the given interface is made; when the factory
    is found, the tool is created and stored into site configuration.

    Custom table name can be provided into configuration file using a setting called
    *pyams_content.config.${table_name}_table_name*.
    """

    if registry is None:
        registry = get_pyramid_registry()
    configuration = ISiteRootToolsConfiguration(site)
    for interface, name in tables:
        configuration.check_table(interface, name, registry)


#
# Checker for required shared tools
#

def check_required_tools(site, tools=REQUIRED_TOOLS, registry=None):
    """Check for required shared tools

    :param site: site root to check for tools
    :param tools: iterable of shared tools to check or create; each occurrence is a tuple of
        two elements containing the tool interface and its default name.
    :param registry: optional registry object

    For each tool which may be registered, the check if it's already defined into current site
    configuration; if not, a factory lookup is done: a custom factory can be defined using a
    setting named *pyams_content.config.{name}_factory*, where name is the default tool name;
    otherwise, a default object factory lookup for the given interface is made; when the factory
    is found, the tool is created and stored into site configuration.

    Custom shared tool name can be provided into configuration file using a setting called
    *pyams_content.config.${table_name}_tool_name*.
    """

    if registry is None:
        registry = get_pyramid_registry()
    configuration = ISiteRootToolsConfiguration(site)
    for interface, name in tools:
        configuration.check_tool(interface, name, registry)


def get_required_indexes():
    """Get list of required indexes based on lexicon settings"""
    indexes = REQUIRED_INDEXES
    registry = get_pyramid_registry()
    for code, language in map(lambda x: x.split(':'),
                              registry.settings.get('pyams_content.lexicon.languages',
                                                    'en:english').split()):
        indexes.append(('title:{0}'.format(code), I18nTextIndexWithInterface, {
            'language': code,
            'interface': IBaseContent,
            'discriminator': 'title',
            'lexicon': lambda: get_fulltext_lexicon(language)
        }))
    return indexes


@utility_config(name='PyAMS content', provides=ISiteGenerations)
class WebsiteGenerationsChecker:
    """PyAMS content package generations checker"""

    order = 100
    generation = 1

    def evolve(self, site, current=None):
        """Check for required utilities, tables and tools"""
        check_required_utilities(site, REQUIRED_UTILITIES)
        check_required_indexes(site, get_required_indexes())
        check_required_tables(site, REQUIRED_TABLES)
        check_required_tools(site, REQUIRED_TOOLS)

        if not current:
            current = 1
        for generation in range(current, self.generation):
            module_name = 'pyams_content.generations.evolve{}'.format(generation)
            module = import_module(module_name)
            module.evolve(site)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.glossary main module

This module defines several helper functions which are used to update glossary.
"""

import logging
import pickle

from ahocorasick import Automaton

from pyams_content.component.thesaurus import ITagsManager
from pyams_thesaurus.interfaces.term import STATUS_PUBLISHED
from pyams_thesaurus.interfaces.thesaurus import IThesaurus
from pyams_utils.cache import get_cache
from pyams_utils.registry import query_utility


__docformat__ = 'restructuredtext'


LOGGER = logging.getLogger('PyAMS (content)')

GLOSSARY_CACHE_NAME = 'persistent'
GLOSSARY_CACHE_REGION = 'persistent'
GLOSSARY_CACHE_NAMESPACE = 'PyAMS::glossary'
GLOSSARY_CACHE_KEY = 'automaton'


def get_glossary_automaton(root):
    """Generate and store glossary automaton"""
    # generate automaton
    tags_manager = ITagsManager(root)
    if not tags_manager.enable_glossary:
        return None
    thesaurus = query_utility(IThesaurus, name=tags_manager.glossary_thesaurus_name)
    if thesaurus is None:
        return None
    LOGGER.debug("Building glossary automaton...")
    automaton = Automaton()
    for term in thesaurus.terms.values():
        if term.status == STATUS_PUBLISHED:
            automaton.add_word(term.label, term.label)
    automaton.make_automaton()
    LOGGER.debug("Automaton built with {} terms".format(len(automaton)))
    # store automaton items
    glossary_cache = get_cache(GLOSSARY_CACHE_NAME, GLOSSARY_CACHE_REGION,
                               GLOSSARY_CACHE_NAMESPACE)
    glossary_cache.set_value(GLOSSARY_CACHE_KEY, pickle.dumps(automaton))
    return automaton


def reset_glossary_automaton():
    """Re-initialize glossary automaton"""
    glossary_cache = get_cache(GLOSSARY_CACHE_NAME, GLOSSARY_CACHE_REGION,
                               GLOSSARY_CACHE_NAMESPACE)
    glossary_cache.clear()



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.video.skin module

This module defines default external video paragraph renderer.
"""

from pyams_content.component.video.interfaces import IExternalVideoParagraph
from pyams_content.component.video.skin.interfaces import IExternalVideoRenderer
from pyams_content.feature.renderer import DefaultContentRenderer, IContentRenderer
from pyams_portal.interfaces import DEFAULT_RENDERER_NAME
from pyams_layer.interfaces import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config

__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(name=DEFAULT_RENDERER_NAME,
                required=(IExternalVideoParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/video-default.pt', layer=IPyAMSLayer)
class ExternalVideoParagraphDefaultRenderer(DefaultContentRenderer):
    """External video paragraph default renderer"""

    label = _("External video renderer (default)")

    video_renderer = None

    def update(self):
        super().update()
        provider = self.context.get_provider()
        if provider is None:
            return ''
        registry = self.request.registry
        renderer = registry.queryMultiAdapter((self.context.settings, self.request),
                                              IExternalVideoRenderer)
        if renderer is not None:
            renderer.update()
        self.video_renderer = renderer

    def render_video(self):
        """Video renderer"""
        if self.video_renderer is None:
            return ''
        return self.video_renderer.render()



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.root.zmi.search module

This module defines components which are used to handle search on site root.
"""

__docformat__ = 'restructuredtext'

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import Any, Contains, Eq, Ge, Le
from pyramid.interfaces import IView
from pyramid.view import view_config
from zope.interface import Interface, implementer
from zope.intid import IIntIds
from zope.schema import Choice, TextLine
from zope.schema.vocabulary import getVocabularyRegistry

from pyams_catalog.query import CatalogResultSet
from pyams_content.component.thesaurus import ICollectionsManager, ITagsManager, IThemesManager
from pyams_content.root.zmi.dashboard import BaseSiteRootDashboardTable, SiteRootDashboardView
from pyams_content.shared.common import SHARED_CONTENT_TYPES_VOCABULARY
from pyams_content.shared.common.zmi.search import SharedToolAdvancedSearchForm, \
    SharedToolAdvancedSearchResultsView, SharedToolAdvancedSearchView, \
    SharedToolQuickSearchPageView, SharedToolQuickSearchView, get_json_search_results, \
    get_quick_search_params
from pyams_content.zmi.interfaces import IAllDashboardMenu
from pyams_form.field import Fields
from pyams_form.interfaces.form import IFormFields, IGroup
from pyams_i18n.interfaces import INegotiator
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_security.schema import PrincipalField
from pyams_sequence.interfaces import ISequentialIntIds
from pyams_sequence.workflow import get_last_version
from pyams_site.interfaces import ISiteRoot
from pyams_table.interfaces import IValues
from pyams_thesaurus.schema import ThesaurusTermsListField
from pyams_thesaurus.zmi.widget import ThesaurusTermsTreeFieldWidget
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.list import unique, unique_iter
from pyams_utils.registry import get_utility
from pyams_utils.schema import DatetimesRangeField
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import FormGroupSwitcher
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import IInnerTable
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

from pyams_content import _


#
# Quick search components
#

@implementer(IView)
class SiteRootQuickSearchResultsTable(BaseSiteRootDashboardTable):
    """Site root quick search results table"""


@adapter_config(required=(ISiteRoot, IPyAMSLayer, SiteRootQuickSearchResultsTable),
                provides=IValues)
class SiteRootQuickSearchResultsValues(ContextRequestViewAdapter):
    """Site root quick search results table values adapter"""

    @property
    def values(self):
        """Table values getter"""
        catalog = get_utility(ICatalog)
        query = self.request.params.get('query', '').strip()
        if not query:
            return ()
        sequence = get_utility(ISequentialIntIds)
        query = query.lower().replace('*', '')
        if query.startswith('+'):
            params = Eq(catalog['oid'], sequence.get_full_oid(query))
        else:
            vocabulary = getVocabularyRegistry().get(self.context,
                                                     SHARED_CONTENT_TYPES_VOCABULARY)
            params = Any(catalog['content_type'], vocabulary.by_value.keys())
            params &= get_quick_search_params(query, self.request, catalog, sequence)
        yield from unique_iter(map(get_last_version,
                                   CatalogResultSet(CatalogQuery(catalog).query(
                                       params, sort_index='modified_date', reverse=True))))


@view_config(name='quick-search.json',
             context=ISiteRoot, request_type=IPyAMSLayer,
             permission=VIEW_SYSTEM_PERMISSION, renderer='json', xhr=True)
def shared_tool_quick_search_view(request):
    """Shared tool quick search view"""
    results = SiteRootQuickSearchResultsTable(request.context, request)
    return get_json_search_results(request, results)


@adapter_config(name='quick-search',
                required=(ISiteRoot, IAdminLayer, SiteRootDashboardView),
                provides=IInnerTable)
class SiteRootQuickSearchView(SharedToolQuickSearchView):
    """Shared tool quick search view"""

    @property
    def legend(self):
        """Legend getter"""
        translate = self.request.localizer.translate
        return translate(_("Between all contents"))


@pagelet_config(name='quick-search.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootQuickSearchPageView(SharedToolQuickSearchPageView):
    """Shared tool quick search page view"""

    table_class = SiteRootQuickSearchResultsTable


#
# Advanced search components
#

@viewlet_config(name='advanced-search.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IAllDashboardMenu, weight=40,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootAdvancedSearchMenu(NavigationMenuItem):
    """Site root advanced search menu"""

    label = _("Advanced search")
    href = "#advanced-search.html"


class ISiteRootAdvancedSearchQuery(Interface):
    """Site root advanced search query interface"""

    query = TextLine(title=_("Search text"),
                     description=_("Entered text will be search in title only"),
                     required=False)

    owner = PrincipalField(title=_("Owner"),
                           required=False)

    content_type = Choice(title=_("Content type"),
                          vocabulary=SHARED_CONTENT_TYPES_VOCABULARY,
                          required=False)

    created = DatetimesRangeField(title=_("Creation date"),
                                  required=False)

    modified = DatetimesRangeField(title=_("Modification date"),
                                   required=False)

    tags = ThesaurusTermsListField(title=_("Tags"),
                                   required=False)

    themes = ThesaurusTermsListField(title=_("Themes"),
                                     required=False)

    collections = ThesaurusTermsListField(title=_("Collections"),
                                          required=False)


class SiteRootAdvancedSearchForm(SharedToolAdvancedSearchForm):
    """Site root advanced search form"""


@adapter_config(required=(Interface, IAdminLayer, SiteRootAdvancedSearchForm),
                provides=IFormFields)
def site_root_advanced_search_form_fields(context, request, form):
    """Site root advanced search form fields getter"""
    return Fields(ISiteRootAdvancedSearchQuery).omit('tags', 'themes', 'collections')


class BaseThesaurusTermsSearchGroup(FormGroupSwitcher):
    """Base thesaurus terms search group"""

    fieldname = None
    manager = None

    def __new__(cls, context, request, view):
        manager = cls.manager(request.root)
        if not manager.thesaurus_name:
            return None
        return FormGroupSwitcher.__new__(cls)

    @property
    def fields(self):
        """Fields getter"""
        fields = Fields(ISiteRootAdvancedSearchQuery).select(self.fieldname)
        fields[self.fieldname].widget_factory = ThesaurusTermsTreeFieldWidget
        return fields

    label_css_class = 'hidden'
    input_css_class = 'col-12'

    def update_widgets(self, prefix=None, use_form_mode=True):
        super().update_widgets(prefix, use_form_mode)
        terms = self.widgets.get(self.fieldname)
        if terms is not None:
            manager = self.manager(self.request.root)
            terms.thesaurus_name = manager.thesaurus_name
            terms.extract_name = manager.extract_name


@adapter_config(name='tags',
                required=(ISiteRoot, IAdminLayer, SiteRootAdvancedSearchForm),
                provides=IGroup)
class SiteRootAdvancedSearchFormTagsGroup(BaseThesaurusTermsSearchGroup):
    """Site root advanced search form tags group"""

    legend = _("Tags")

    fieldname = 'tags'
    manager = ITagsManager

    weight = 10


@adapter_config(name='themes',
                required=(ISiteRoot, IAdminLayer, SiteRootAdvancedSearchForm),
                provides=IGroup)
class SiteRootAdvancedSearchFormThemesGroup(BaseThesaurusTermsSearchGroup):
    """Site root advanced search form themes group"""

    legend = _("Themes")

    fieldname = 'themes'
    manager = IThemesManager

    weight = 20


@adapter_config(name='collections',
                required=(ISiteRoot, IAdminLayer, SiteRootAdvancedSearchForm),
                provides=IGroup)
class SharedToolAdvancedSearchFormCollectionsGroup(BaseThesaurusTermsSearchGroup):
    """Shared tool advanced search form collections group"""

    legend = _("Collections")

    fieldname = 'collections'
    manager = ICollectionsManager

    weight = 30


@pagelet_config(name='advanced-search.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootAdvancedSearchView(SharedToolAdvancedSearchView):
    """Site root advanced search view"""

    search_form = SiteRootAdvancedSearchForm


class SiteRootAdvancedSearchResultsTable(BaseSiteRootDashboardTable):
    """Site root advanced search results table"""


@adapter_config(required=(ISiteRoot, IPyAMSLayer, SiteRootAdvancedSearchResultsTable),
                provides=IValues)
class SiteRootAdvancedSearchResultsValues(ContextRequestViewAdapter):
    """Site root advanced search results values"""

    def get_params(self, data):
        """Extract catalog query params from incoming request"""
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = Any(catalog['content_type'], vocabulary.by_value.keys())
        query = data.get('query')
        if query:
            sequence = get_utility(ISequentialIntIds)
            if query.startswith('+'):
                params &= Eq(catalog['oid'], sequence.get_full_oid(query))
            else:
                query_params = Eq(catalog['oid'], sequence.get_full_oid(query))
                negotiator = get_utility(INegotiator)
                for lang in {self.request.registry.settings.get('pyramid.default_locale_name',
                                                                'en'),
                             self.request.locale_name,
                             negotiator.server_language} | negotiator.offered_languages:
                    index_name = 'title:{0}'.format(lang)
                    if index_name in catalog:
                        index = catalog[index_name]
                        if index.check_query(query):
                            query_params |= Contains(index,
                                                     ' and '.join((w+'*' for w in query.split())))
                params &= query_params
        if data.get('owner'):
            params &= Eq(catalog['role:owner'], data['owner'])
        if data.get('content_type'):
            params &= Eq(catalog['content_type'], data['content_type'])
        created_after, created_before = data.get('created', (None, None))
        if created_after:
            params &= Ge(catalog['created_date'], created_after)
        if created_before:
            params &= Le(catalog['created_date'], created_before)
        modified_after, modified_before = data.get('modified', (None, None))
        if modified_after:
            params &= Ge(catalog['modified_date'], modified_after)
        if modified_before:
            params &= Le(catalog['modified_date'], modified_before)
        if data.get('tags'):
            tags = [intids.register(term) for term in data['tags']]
            params &= Any(catalog['tags'], tags)
        if data.get('themes'):
            tags = [intids.register(term) for term in data['themes']]
            params &= Any(catalog['themes'], tags)
        if data.get('collections'):
            tags = [intids.register(term) for term in data['collections']]
            params &= Any(catalog['collections'], tags)
        return params

    @property
    def values(self):
        """Shared tool advanced search results values getter"""
        form = SiteRootAdvancedSearchForm(self.context, self.request)
        form.update()
        data, _errors = form.extract_data()
        params = self.get_params(data)
        catalog = get_utility(ICatalog)
        yield from unique_iter(map(get_last_version,
                                   CatalogResultSet(CatalogQuery(catalog).query(
                                       params, sort_index='modified_date', reverse=True))))


@pagelet_config(name='advanced-search-results.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION, xhr=True)
class SiteRootAdvancedSearchResultsView(SharedToolAdvancedSearchResultsView):
    """Site root advanced search results view"""

    table_class = SiteRootAdvancedSearchResultsTable



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration.portlet module

This module provides illustration portlet.
"""

from pyams_content.component.illustration.portlet.interfaces import IIllustrationPortletSettings
from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
from pyams_utils.factory import factory_config


__docformat__ = 'restructuredtext'

from pyams_content import _


ILLUSTRATION_PORTLET_NAME = 'pyams_content.portlet.illustration'
ILLUSTRATION_ICON_CLASS = 'far fa-image'


@factory_config(IIllustrationPortletSettings)
class IllustrationPortletSettings(PortletSettings):
    """Illustration portlet settings"""


@portlet_config(permission=None)
class IllustrationPortlet(Portlet):
    """Illustration portlet"""

    name = ILLUSTRATION_PORTLET_NAME
    label = _("Illustration")

    settings_factory = IIllustrationPortletSettings
    toolbar_css_class = ILLUSTRATION_ICON_CLASS



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.html module

This module provides several HTML renderers:
 - 'oid_to_href' is used to replace an HTML link URL using oid://xxx syntax to a real
   link to the specified target; the link is removed if the target is not published or can't be
   found; please note also that the link is always targetting the currently published version,
   if any;
 - 'remove_tags' is used to remove all tags from incoming HTML content.
"""

from pyquery import PyQuery
from pyramid.interfaces import IRequest

from pyams_sequence.interfaces import ISequentialIntIds
from pyams_sequence.reference import get_reference_target
from pyams_utils.adapter import ContextRequestAdapter, adapter_config
from pyams_utils.interfaces.text import IHTMLRenderer
from pyams_utils.registry import get_utility
from pyams_utils.request import get_display_context
from pyams_utils.text import text_to_html
from pyams_utils.url import relative_url
from pyams_workflow.interfaces import IWorkflowPublicationInfo


__docformat__ = 'restructuredtext'


@adapter_config(name='oid_to_href',
                required=(str, IRequest),
                provides=IHTMLRenderer)
class OIDHTMLRenderer(ContextRequestAdapter):
    """An HTML renderer converting all "oid://" URLs to internal relative links"""

    def render(self):
        """Render oid:// links"""
        request = self.request
        context = get_display_context(request)
        html = PyQuery(f'<div>{self.context}</div>')
        sequence = get_utility(ISequentialIntIds)
        for link in html('a[href]'):
            href = link.attrib['href']
            if href.startswith('oid://'):
                oid = sequence.get_full_oid(href.split('//', 1)[1])
                target = get_reference_target(oid, request=request)
                if target is not None:
                    publication_info = IWorkflowPublicationInfo(target, None)
                    if (publication_info is not None) and \
                            publication_info.is_visible(request):
                        link.attrib['href'] = relative_url(target, request, context)
                        continue
                # invalid link => remove href!
                del link.attrib['href']
        return html.html()


@adapter_config(name='remove_tags',
                required=(str, IRequest),
                provides=IHTMLRenderer)
class TagsRemoveHTMLRenderer(ContextRequestAdapter):
    """Remove all tags from HTML code"""

    def render(self):
        """Render HTML source without tags"""
        html = PyQuery('<div>{}</div>'.format(self.context))
        return text_to_html(html.text())



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.extfile.zmi.paragraph module

This module defines components which are used to handle paragraphs
which are also associations containers.
"""

from pyams_content.component.association import IAssociationContainer, IAssociationContainerTarget
from pyams_content.component.extfile import EXTAUDIO_ICON_CLASS, EXTFILE_ICON_CLASS, \
    EXTIMAGE_ICON_CLASS, EXTVIDEO_ICON_CLASS, IExtAudio, IExtFile, IExtImage, IExtVideo
from pyams_content.component.paragraph.zmi.container import ParagraphTitleToolbarItemMixin
from pyams_content.component.paragraph.zmi.interfaces import IParagraphContainerFullTable, \
    IParagraphTitleToolbar
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer


__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='external-files',
                context=IAssociationContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerFullTable, manager=IParagraphTitleToolbar,
                weight=50)
class ExternalFilesTitleToolbarViewlet(ParagraphTitleToolbarItemMixin):
    """External files title toolbar viewlet"""

    icon_class = EXTFILE_ICON_CLASS
    icon_hint = _("Internal files")

    target_intf = IAssociationContainer
    item_intf = IExtFile


@viewlet_config(name='external-images',
                context=IAssociationContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerFullTable, manager=IParagraphTitleToolbar,
                weight=60)
class ExternalImagesTitleToolbarViewlet(ParagraphTitleToolbarItemMixin):
    """External images title toolbar viewlet"""

    icon_class = EXTIMAGE_ICON_CLASS
    icon_hint = _("Internal images")

    target_intf = IAssociationContainer
    item_intf = IExtImage


@viewlet_config(name='external-videos',
                context=IAssociationContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerFullTable, manager=IParagraphTitleToolbar,
                weight=70)
class ExternalVideosTitleToolbarViewlet(ParagraphTitleToolbarItemMixin):
    """External files title toolbar viewlet"""

    icon_class = EXTVIDEO_ICON_CLASS
    icon_hint = _("Internal videos")

    target_intf = IAssociationContainer
    item_intf = IExtVideo


@viewlet_config(name='external-audios',
                context=IAssociationContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerFullTable, manager=IParagraphTitleToolbar,
                weight=80)
class ExternalAudiosTitleToolbarViewlet(ParagraphTitleToolbarItemMixin):
    """External audio files title toolbar viewlet"""

    icon_class = EXTAUDIO_ICON_CLASS
    icon_hint = _("Internal audio files")

    target_intf = IAssociationContainer
    item_intf = IExtAudio



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration module

"""

__docformat__ = 'restructuredtext'

from persistent import Persistent
from pyramid.events import subscriber
from zope.container.contained import Contained
from zope.interface import alsoProvides, classImplements
from zope.lifecycleevent import IObjectAddedEvent, IObjectModifiedEvent, ObjectAddedEvent
from zope.location.interfaces import ISublocations
from zope.schema.fieldproperty import FieldProperty
from zope.traversing.interfaces import ITraversable

from pyams_content.component.illustration.interfaces import BASIC_ILLUSTRATION_KEY, \
    IBaseIllustration, IBaseIllustrationTarget, IIllustration, IIllustrationTarget, \
    IIllustrationTargetBase, ILLUSTRATION_KEY, ILLUSTRATION_RENDERERS, ILinkIllustration, \
    ILinkIllustrationTarget, IParagraphIllustration, LINK_ILLUSTRATION_KEY
from pyams_content.component.paragraph import IBaseParagraph
from pyams_content.feature.renderer import RenderedContentMixin, RenderersVocabulary
from pyams_file.interfaces import IFileInfo, IImageFile, IResponsiveImage
from pyams_file.property import I18nFileProperty
from pyams_i18n.interfaces import II18n, INegotiator
from pyams_site.site import BaseSiteRoot
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_pyramid_registry, query_utility
from pyams_utils.request import check_request
from pyams_utils.vocabulary import vocabulary_config


classImplements(BaseSiteRoot, IIllustrationTarget)


@factory_config(IBaseIllustration)
class BasicIllustration(Persistent, Contained):
    """Illustration persistent class"""

    _data = I18nFileProperty(IBaseIllustration['data'])
    title = FieldProperty(IBaseIllustration['title'])
    alt_title = FieldProperty(IBaseIllustration['alt_title'])
    author = FieldProperty(IBaseIllustration['author'])

    @property
    def data(self):
        """Data property getter"""
        return self._data

    @data.setter
    def data(self, value):
        """Data property setter"""
        self._data = value
        for data in (self._data or {}).values():
            if IImageFile.providedBy(data):
                alsoProvides(data, IResponsiveImage)

    @data.deleter
    def data(self):
        """Data property deleter"""
        del self._data

    def has_data(self):
        if not self._data:
            return False
        for data in self._data.values():
            if bool(data):
                return True
        return False


@factory_config(provided=IIllustration)
class Illustration(RenderedContentMixin, BasicIllustration):
    """Illustration persistent class"""

    description = FieldProperty(IIllustration['description'])
    renderer = FieldProperty(IIllustration['renderer'])


@adapter_config(required=IBaseIllustrationTarget,
                provides=IIllustration)
def basic_illustration_factory(context):
    """Basic illustration factory"""

    def illustration_callback(illustration):
        get_pyramid_registry().notify(ObjectAddedEvent(illustration, context,
                                                       illustration.__name__))

    return get_annotation_adapter(context, BASIC_ILLUSTRATION_KEY, IBaseIllustration,
                                  name='++illustration++',
                                  callback=illustration_callback)


@adapter_config(required=IIllustrationTarget,
                provides=IIllustration)
def illustration_factory(context):
    """Illustration factory"""

    def illustration_callback(illustration):
        if IBaseParagraph.providedBy(context):
            alsoProvides(illustration, IParagraphIllustration)
        get_pyramid_registry().notify(ObjectAddedEvent(illustration, context,
                                                       illustration.__name__))

    return get_annotation_adapter(context, ILLUSTRATION_KEY, IIllustration,
                                  name='++illustration++',
                                  callback=illustration_callback)


@adapter_config(required=ILinkIllustrationTarget,
                provides=ILinkIllustration)
@adapter_config(name='link',
                required=ILinkIllustrationTarget,
                provides=IIllustration)
def link_illustration_factory(context):
    """Link illustration factory"""

    def illustration_callback(illustration):
        get_pyramid_registry().notify(ObjectAddedEvent(illustration, context,
                                                       illustration.__name__))

    return get_annotation_adapter(context, LINK_ILLUSTRATION_KEY, IBaseIllustration,
                                  markers=ILinkIllustration,
                                  name='++illustration++link',
                                  callback=illustration_callback)


def update_illustration_properties(illustration):
    """Update missing file properties"""
    request = check_request()
    i18n = query_utility(INegotiator)
    if i18n is not None:
        lang = i18n.server_language
        data = II18n(illustration).get_attribute('data', lang, request)
        if data:
            info = IFileInfo(data)
            info.title = II18n(illustration).get_attribute('title', lang, request)
            info.description = II18n(illustration).get_attribute('alt_title', lang, request)


@subscriber(IObjectAddedEvent, context_selector=IBaseIllustration)
def handle_added_illustration(event):
    """Handle added illustration"""
    illustration = event.object
    update_illustration_properties(illustration)


@subscriber(IObjectModifiedEvent, context_selector=IBaseIllustration)
def handle_modified_illustration(event):
    """Handle modified illustration"""
    illustration = event.object
    update_illustration_properties(illustration)


@adapter_config(name='illustration',
                required=IIllustrationTargetBase,
                provides=ITraversable)
class IllustrationNamespace(ContextAdapter):
    """++illustration++ namespace adapter"""

    def traverse(self, name, furtherpath=None):
        registry = get_pyramid_registry()
        return registry.queryAdapter(self.context, IIllustration, name=name)


@adapter_config(name='illustration',
                required=IIllustrationTargetBase,
                provides=ISublocations)
class IllustrationSublocations(ContextAdapter):
    """Illustration sub-locations adapter"""

    def sublocations(self):
        """Sub-locations iterator"""
        registry = get_pyramid_registry()
        for name, adapter in registry.getAdapters((self,), IBaseIllustration):
            yield adapter


@vocabulary_config(name=ILLUSTRATION_RENDERERS)
class IllustrationRenderersVocabulary(RenderersVocabulary):
    """Illustration renderers vocabulary"""

    content_interface = IIllustration


#
# Custom image file to illustration adapter
#

@adapter_config(required=IImageFile,
                provides=IBaseIllustration)
class VirtualIllustration:
    """Virtual illustration based on image file"""

    title = None
    alt_title = None
    author = None

    def __init__(self, source):
        self.source = source

    @property
    def data(self):
        return self.source

    def has_data(self):
        return bool(self.source)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from pyams_content.shared.view.portlet.skin import IViewItemTargetURL
from pyams_layer.interfaces import IPyAMSLayer
from pyams_utils.adapter import ContextRequestAdapter, adapter_config


# @adapter_config(required=(IWfLogo, IPyAMSLayer),
#                 provides=IViewItemTargetURL)
# class LogoViewItemTarget(ContextRequestAdapter):
#     """Logo view item target getter"""
#
#     @property
#     def target(self):
#         """Logo target getter"""
#         return self.context.target
#
#     @property
#     def url(self):
#         """Logo URL getter"""
#         return self.context.url



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.portlet.interfaces module

This module defines interfaces of shared contents portlets settings.
"""

__docformat__ = 'restructuredtext'

from zope.contentprovider.interfaces import IContentProvider
from zope.schema import Bool

from pyams_i18n.schema import I18nTextLineField
from pyams_portal.interfaces import IPortletSettings

from pyams_content import _


TITLE_PORTLET_NAME = 'pyams_content.portlet.title'


class ISharedContentTitlePortletSettings(IPortletSettings):
    """Shared content title portlet settings interface"""

    display_publication_date = Bool(title=_("Display publication date?"),
                                    description=_("If 'yes', content publication date will be "
                                                  "displayed; the selected publication date is "
                                                  "those which is selected while publishing the "
                                                  "content"),
                                    required=True,
                                    default=False)

    publication_date_prefix = I18nTextLineField(title=_("Publication date prefix"),
                                                description=_("This text will be displayed "
                                                              "before publication date"),
                                                required=False)

    display_specificities = Bool(title=_("Display specificities?"),
                                 description=_("If 'no', specific content's information will not "
                                               "be displayed..."),
                                 required=True,
                                 default=True)


HEADER_PORTLET_NAME = 'pyams_content.portlet.header'


class ISharedContentHeaderPortletSettings(IPortletSettings):
    """Shared content header portlet settings"""

    display_illustration = Bool(title=_("Display illustration?"),
                                description=_("If enabled, and if a header illustration is "
                                              "associated with the content, it's selected ratio "
                                              "may vary according to the selected renderer"),
                                required=True,
                                default=True)

    display_breadcrumbs = Bool(title=_("Display breadcrumbs?"),
                               required=True,
                               default=True)

    display_title = Bool(title=_("Display title?"),
                         required=True,
                         default=True)

    display_tags = Bool(title=_("Display tags?"),
                        required=True,
                        default=True)

    display_header = Bool(title=_("Display header?"),
                          required=True,
                          default=True)

    display_publication_date = Bool(title=_("Display publication date?"),
                                    description=_("If 'yes', content publication date will be "
                                                  "displayed; the selected publication date is "
                                                  "those which is selected while publishing the "
                                                  "content"),
                                    required=True,
                                    default=False)

    publication_date_prefix = I18nTextLineField(title=_("Publication date prefix"),
                                                description=_("This text will be displayed "
                                                              "before publication date"),
                                                required=False)

    display_alerts = Bool(title=_("Display alerts?"),
                          description=_("If 'no', alerts which are linked to a specific content "
                                        "will not be displayed"),
                          required=True,
                          default=True)

    display_specificities = Bool(title=_("Display specificities?"),
                                 description=_("If 'no', specific content's information will not "
                                               "be displayed..."),
                                 required=True,
                                 default=True)


SPECIFICITIES_PORTLET_NAME = 'pyams_content.portlet.specificities'


class ISharedContentSpecificitiesPortletSettings(IPortletSettings):
    """Shared content specificities portlet settings interface"""


class ISpecificitiesRenderer(IContentProvider):
    """Specificities renderer interface"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_fields.zmi.mail module

Management interface components for 'mailto' form handler.
"""

from pyams_content.component.fields.handler.interfaces import IMailtoHandlerInfo
from pyams_content.shared.common.zmi import ISharedContentPropertiesMenu
from pyams_fields.interfaces import IFormHandlersInfo, IFormHandlersTarget
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IFormContent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='mailto-handler.menu',
                context=IFormHandlersTarget, layer=IAdminLayer,
                manager=ISharedContentPropertiesMenu, weight=10,
                permission=VIEW_SYSTEM_PERMISSION)
class MailtoFormHandlerSettingsMenu(NavigationMenuItem):
    """Mailto form handler settings menu"""

    label = _("Mailto handler settings")
    href = '#mailto-handler.html'

    def __new__(cls, context, request, view, manager):
        handlers = IFormHandlersInfo(context, None)
        if handlers is None:
            return None
        mailto_info = IMailtoHandlerInfo(handlers, None)
        if mailto_info is None:
            return None
        return NavigationMenuItem.__new__(cls)


@ajax_form_config(name='mailto-handler.html',
                  context=IFormHandlersTarget, layer=IPyAMSLayer)
class MailtoFormHandlerSettingsEditForm(AdminEditForm):
    """Mailto form handlers settings form"""

    title = _("Mailto notifications")
    legend = _("Mailto form handler settings")

    fields = Fields(IMailtoHandlerInfo)


@adapter_config(required=(IFormHandlersTarget, IPyAMSLayer, MailtoFormHandlerSettingsEditForm),
                provides=IFormContent)
def form_handlers_settings_edit_form_content(context, request, form):
    """Form handlers settings edit form content getter"""
    handlers = IFormHandlersInfo(context)
    return IMailtoHandlerInfo(handlers, None)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph.skin.html module

"""

from persistent import Persistent
from pygments.lexers import get_lexer_by_name
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.illustration import IIllustration
from pyams_content.component.paragraph.interfaces.html import IHTMLParagraph, IRawParagraph
from pyams_content.component.paragraph.skin.interfaces.html import IHTMLParagraphRendererSettings
from pyams_content.feature.renderer import DefaultContentRenderer, \
    IContentRenderer
from pyams_portal.interfaces import DEFAULT_RENDERER_NAME
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils import library
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.fanstatic import ExternalResource
from pyams_utils.interfaces.pygments import IPygmentsCodeConfiguration
from pyams_utils.interfaces.text import IHTMLRenderer
from pyams_utils.pygments import render_source


__docformat__ = 'restructuredtext'

from pyams_content import _


#
# Raw HTML code renderer
#

@adapter_config(name=DEFAULT_RENDERER_NAME,
                required=(IRawParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/raw.pt', layer=IPyAMSLayer)
class RawParagraphDefaultRenderer(DefaultContentRenderer):
    """Raw paragraph default renderer"""

    label = _("HTML source code (default)")


@adapter_config(name='source-code',
                required=(IRawParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/raw-code.pt', layer=IPyAMSLayer)
class RawParagraphSourceCodeRenderer(DefaultContentRenderer):
    """Raw paragraph source code renderer"""

    label = _("Formatted source code")
    weight = 10

    settings_interface = IPygmentsCodeConfiguration

    @property
    def editor_filename(self):
        """ACE editor mode getter"""
        settings = self.settings
        if (settings is not None) and (settings.lexer not in (None, 'auto')):
            lexer = get_lexer_by_name(settings.lexer)
            return f"body.{lexer.filenames[0].rsplit('.', 1)[-1]}"
        return 'body.html'

    @property
    def resources(self):
        """Fanstatic resources getter"""
        settings = self.settings
        path = f'get-pygments-style.css?style={settings.style}'
        resource = library.known_resources.get(path)
        if resource is None:
            resource = ExternalResource(library, path, resource_type='css')
            if library.library_nr is None:
                library.init_library_nr()
        yield resource

    @property
    def body(self):
        """Formatted body getter"""
        body = II18n(self.context).query_attribute('body', request=self.request)
        if not body:
            return ''
        return render_source(body, self.settings)


@adapter_config(name='rest',
                required=(IRawParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/raw-code.pt', layer=IPyAMSLayer)
class RawParagraphRestRenderer(DefaultContentRenderer):
    """Raw paragraph ReStructured text renderer"""

    label = _("ReStructured text")
    weight = 20

    editor_filename = 'body.rst'

    @property
    def body(self):
        """Formatted body getter"""
        body = II18n(self.context).query_attribute('body', request=self.request)
        if not body:
            return ''
        renderer = self.request.registry.queryMultiAdapter((body, self.request),
                                                           IHTMLRenderer, name='rest')
        if renderer is None:
            return ''
        return renderer.render()


@adapter_config(name='markdown',
                required=(IRawParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/raw-code.pt', layer=IPyAMSLayer)
class RawParagraphMarkdownRenderer(DefaultContentRenderer):
    """Raw paragraph Markdown text renderer"""

    label = _("Markdown text")
    weight = 30

    editor_filename = 'body.md'

    @property
    def body(self):
        """Formatted body getter"""
        body = II18n(self.context).query_attribute('body', request=self.request)
        if not body:
            return ''
        renderer = self.request.registry.queryMultiAdapter((body, self.request),
                                                           IHTMLRenderer, name='markdown')
        if renderer is None:
            return ''
        return renderer.render()


#
# HTML rich text renderer
#

@factory_config(IHTMLParagraphRendererSettings)
class HTMLParagraphRendererSettings(Persistent, Contained):
    """HTML paragraph renderer settings"""

    xs_horizontal_padding = FieldProperty(IHTMLParagraphRendererSettings['xs_horizontal_padding'])
    sm_horizontal_padding = FieldProperty(IHTMLParagraphRendererSettings['sm_horizontal_padding'])
    md_horizontal_padding = FieldProperty(IHTMLParagraphRendererSettings['md_horizontal_padding'])
    lg_horizontal_padding = FieldProperty(IHTMLParagraphRendererSettings['lg_horizontal_padding'])
    xl_horizontal_padding = FieldProperty(IHTMLParagraphRendererSettings['xl_horizontal_padding'])

    def has_padding(self):
        """Check if any padding is defined"""
        return any((self.xs_horizontal_padding,
                    self.sm_horizontal_padding,
                    self.md_horizontal_padding,
                    self.lg_horizontal_padding,
                    self.xl_horizontal_padding))

    def get_padding(self):
        """Paddings getter"""
        result = []
        for device in ('xs', 'sm', 'md', 'lg', 'xl'):
            padding = getattr(self, f'{device}_horizontal_padding')
            if padding is not None:
                result.append(f'col-{device}-{12 - padding*2} offset-{device}-{padding}')
        return ' '.join(result)


@adapter_config(name=DEFAULT_RENDERER_NAME,
                required=(IHTMLParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/html.pt', layer=IPyAMSLayer)
class HTMLParagraphRenderer(DefaultContentRenderer):
    """HTML paragraph default renderer"""

    label = _("Rich text (default)")

    settings_interface = IHTMLParagraphRendererSettings

    illustration = None
    illustration_renderer = None

    def update(self):
        super().update()
        illustration = IIllustration(self.context)
        if illustration.has_data():
            self.illustration = illustration
            self.illustration_renderer = illustration.get_renderer(self.request)
            self.illustration_renderer.update()



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi.reference module

This module provides management components which are used to handle
internal components which can be used to feed views.
"""
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_sequence.interfaces import IInternalReferencesList
from pyams_skin.interfaces.viewlet import IFormHeaderViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='internal-references.menu',
                context=IInternalReferencesList, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=300,
                permission=VIEW_SYSTEM_PERMISSION)
class InternalReferencesMenu(NavigationMenuItem):
    """Internal references menu"""

    label = _("Internal references")
    href = '#internal-references.html'

    def __new__(cls, context, request, view, manager=None):
        references = IInternalReferencesList(context, None)
        if not getattr(references, 'use_references_for_views', True):
            return None
        return NavigationMenuItem.__new__(cls)


@ajax_form_config(name='internal-references.html',
                  context=IInternalReferencesList, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class InternalReferencesEditForm(AdminEditForm):
    """Internal references edit form"""

    title = _("Internal references")
    legend = _("Internal references settings")

    fields = Fields(IInternalReferencesList)


@viewlet_config(name='internal-references.help',
                context=IInternalReferencesList, layer=IAdminLayer, view=InternalReferencesEditForm,
                manager=IFormHeaderViewletManager, weight=1)
class InternalReferencesEditFormHelp(AlertMessage):
    """Internal references edit form help"""

    status = 'info'

    _message = _("Selected contents can be used to feed views which are using their "
                 "context internal references.")
    message_renderer = 'markdown'



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.extfile.interfaces module

This module defines interfaces which are used to handle external files.
"""

from zope.interface import Interface
from zope.schema import Choice, TextLine

from pyams_content.component.association import IAssociationContainerTarget, IAssociationItem
from pyams_file.schema import I18nAudioField, I18nFileField, I18nThumbnailImageField, \
    I18nVideoField
from pyams_i18n.interfaces import BASE_LANGUAGES_VOCABULARY_NAME
from pyams_i18n.schema import I18nTextField, I18nTextLineField


__docformat__ = 'restructuredtext'

from pyams_content import _


EXTFILE_CONTAINER_KEY = 'pyams_content.extfile'
EXTFILE_LINKS_CONTAINER_KEY = 'pyams_content.extfile.links'

EXTFILE_FACTORIES_VOCABULARY = 'pyams_content.extfile.factories'


class IBaseExtFile(IAssociationItem):
    """Base external file interface"""

    title = I18nTextLineField(title=_("Download link label"),
                              description=_("Label of download link, as shown in front-office"),
                              required=False)

    description = I18nTextField(title=_("Description"),
                                description=_("File description displayed by front-office "
                                              "template"),
                                required=False)

    author = TextLine(title=_("Author"),
                      description=_("Name of document's author"),
                      required=False)

    language = Choice(title=_("Language"),
                      description=_("File's content language"),
                      vocabulary=BASE_LANGUAGES_VOCABULARY_NAME,
                      required=False)

    filename = TextLine(title=_("Save file as..."),
                        description=_("Name under which the file will be saved"),
                        required=False)


EXTFILE_ICON_CLASS = 'far fa-file-alt'
EXTFILE_ICON_HINT = _("Standard file")


class IExtFile(IBaseExtFile):
    """Generic external file interface"""

    data = I18nFileField(title=_("File data"),
                         description=_("File content"),
                         required=True)


class IExtMedia(IExtFile):
    """External media file interface"""


EXTIMAGE_ICON_CLASS = 'far fa-image'
EXTIMAGE_ICON_HINT = _("External image")


class IExtImage(IExtMedia):
    """External image file interface"""

    data = I18nThumbnailImageField(title=_("Image data"),
                                   description=_("Image content"),
                                   required=True)


EXTVIDEO_ICON_CLASS = 'fas fa-film'
EXTVIDEO_ICON_HINT = _("Video")


class IExtVideo(IExtMedia):
    """External video file interface"""

    data = I18nVideoField(title=_("Video data"),
                          description=_("Video content"),
                          required=True)


EXTAUDIO_ICON_CLASS = 'fas fa-headphones'
EXTAUDIO_ICON_HINT = _("Audio file")


class IExtAudio(IExtMedia):
    """External audio file interface"""

    data = I18nAudioField(title=_("Audio data"),
                          description=_("Audio file content"),
                          required=True)


class IExtFileContainerTarget(IAssociationContainerTarget):
    """External files container marker interface"""


#
# External files management
#

EXTFILE_MANAGER_INFO_KEY = 'pyams_content.extfile.manager'


class IExtFileManagerInfo(Interface):
    """External file manager interface"""

    default_title_prefix = I18nTextLineField(title=_("Default title prefix"),
                                             description=_("If used, this prefix will be "
                                                           "automatically added to download "
                                                           "link's label of all files"),
                                             required=False)


class IExtFileManagerTarget(Interface):
    """External files manager target interface"""
